#ifndef _WINC_BUFRFILE
#define _WINC_BUFRFILE

#include <WXP/BufrCode.h>
#include <WXP/BufrData.h>
#include <WXP/BufrParam.h>
#include <WXP/File.h>

namespace WXP {
#define DATA_SIZE 10000
   class BufrFile {
      File file;
      bool opened;
   
      int type;
      int version;
      int size;

      int loc;
      int data_size;
      int data_type;
      int data_subtype;
      File lfile;
      unsigned char data[DATA_SIZE];
      unsigned char *data_ptr;

      int  bpval;                /* Specifies the number of bits per
                                  * gridpoint value used in encoding GRIB */
      int  bpoff;                /* Offset in data stream in bytes */
      int  data_bits;    
      long bit_mask;    
      int  temp_data;

      int loop_ind;
      int beg[20];		// Loop params 
      int end[20];
      int cnt[20];
      int num[20];

      int twidth;       // Temporary changes to width, scale and reference
      int tscale;
      int tref;

      int master_table;

      BufrCode *codes;
      int code_ind;
      int num_codes;
      int max_codes;

      BufrParam *params;
      int param_ind;
      int num_params;
      int num_subset;

      int seq_ind;
      int seq_end;

      int dind;
      int dcode[30];
      int dcodei[30];
      char dcodes[30][200];

      int initClass();
      int searchBUFR();
      int readData( int bytes );
      int readBits( int bpval );
      unsigned char getByte( int loc );
      int seekData( int val );
      int addCode( BufrCode &code );
      int resizeCode( int size );
      int searchCode( int rmaj, int rmin, BufrCode &code );
      int searchCodes( int rmaj, int rmin, int *pcodes );
      int searchDataCode( int rmaj, int rmin, int val, char *str );
      
      public:
      BufrFile();
      ~BufrFile();
      int init();
      int open( const char *filename );
      bool isOpen();
      int close();
      int readHeader();
      int readPDB();
      int readData( BufrData &data );
      /** Get index into parameter list */
      inline int getParamInd(){ return param_ind; }; 
      /** Get loop index */
      inline int getLoop(){ return loop_ind; }; 
      /** Get number of times  to go through the loop */
      inline int getLoopNum(){ return num[loop_ind]; }; 
      /** Get current count in loop */
      inline int getLoopInd(){ return cnt[loop_ind]; }; 
      /** Parameter index of start of loop */
      inline int getLoopBeg(){ return beg[loop_ind]; }; 
      /** Parameter index of end of loop */ 
      inline int getLoopEnd(){ return end[loop_ind]; }; 
      /** Get number of data subsets */
      inline int getNumSubsets(){ return num_subset; }; 
      /** Move to next dataset */
      inline int nextDataset(){ code_ind = 0; return 0; }; 
      int printParams();
      int printCodes();
      int print();

      static int checkType( const char *filename );

      enum FileType {
         NONE,
         WXP,
         LDM,
         RAW
      };
   };
}   

#endif
