#ifndef _WINC_DATE
#define _WINC_DATE

#include <WXP/DateVal.h>
#include <WXP/String.h>
   
namespace WXP {
   class Date : private DateVal {
      int wday;     /**< Day of week */
      int jday;     /**< Julian day */
      long secs;    /**< Total seconds */
      int off;      /**< Second offset from UTC */
      char zone[5]; /**< Timezone descriptive string */
   
      int computeSecs();
   
      public:
      Date();
      Date( int ryear, int rmonth, int rday, int rhour, int rmin, float rsec );
      Date( DateVal &date );
      Date( long secs );
      Date( char *str );
      int operator>( const Date &date );
      int operator<( const Date &date );
      int init();
      bool isSet();
      int set( const char *str );
      int set( int ryear, int rmonth, int rday, int rhour, int rmin, float rsec );
      /** Sets date given another Date class */
      inline int set( Date &rdate ){ *this = rdate; return 1; };
      int set( DateVal &date );
      int set( DateVal &date, int roff, const char *tz );
      int set( long secs );
      int set();
      int getVals( DateVal &date );
      DateVal get();
      int setYear( int val );
      /** Gets the year \return Year (yyyy) */
      inline int getYear(){ return year; };
      int getYear00();
      int setMonth( int val );
      /** Gets the month \return Month */
      inline int getMonth(){ return month; };
      int setDay( int val );
      /** Gets the month \return Day */
      inline int getDay(){ return day; };
      int setHour( int val );
      /** Gets the hour \return Hour */
      inline int getHour(){ return hour; };
      int setMinute( int val );
      /** Gets the minute \return Minute */
      inline int getMinute(){ return min; };
      int setSecond( float val );
      int setFractionalSecond( float val );
      /** Gets the seconds \return Seconds */
      inline float getSecond(){ return sec; };
      int getSeconds();
      bool isLeap();
      int setDays( int days );
      int setTime( int rhour, int rmin, float rsec );
      int setTime( float rsecs );
      int getDays();
      int addYears( int years );
      int addMonths( int months );
      int addDays( int days );
      int addHours( int hours );
      int addMinutes( int mins );
      int addSeconds( float secs );
      int getDayOfWeek();
      int getJulian();
      int setJulian( int val );
      int toString( String &str );
      int toText( String &str );
      int toText( const char *form, String &str );
      int changeTimezone( int roff, const char *tz );
      int adjustMinute( int rmonth, int rday, int rhour, int rmin );
      int adjustMinute( int rday, int rhour, int rmin );
      int adjustMinute( int rhour, int rmin );
      int adjustSecond( int rhour, int rmin, int rsec );
      int adjustHour( int rmonth, int rday, int rhour );
      int adjustHour( int rday, int rhour );
      int adjustHour( int rhour );
      int fix();
      int print();
      int printSeconds();
       
      static long getCurrentSeconds();
      static bool isYearLeap( int year );
      static int setJulian( int year, int jday, int &month, int &day );
      static int getJulian( int year, int month, int day );
   };
}   
   
#endif
