#ifndef _WINC_DATUM
#define _WINC_DATUM

#include <WXP/EarthPoint.h>

namespace WXP {
   class Datum {
      char id[20];
      EarthPoint ept;
      int type;                 /* Type of data */ 
      int index;                /* Index of data */ 
      char format[80];          /* Local Format */
      char data[40];            /* String value for a station */
      float value[5];           /* Value at station */
   
      public:
      Datum();
      int init();
      int initVals();
      int set( float rlat, float rlon, int type, const char *data, const char *format );
      int set( float rlat, float rlon, int type, float val, const char *format );
      int set( float rlat, float rlon, int type, float val1, float val2, const char *format );
      int set( EarthPoint ept, int type, const char *data, const char *format );
      int set( EarthPoint ept, int type, float val, const char *format );
      int set( EarthPoint ept, int type, float val1, float val2, const char *format );
      int setId( const char *rid );
      /** Return station identifier */
      inline const char *getId(){ return id; };
      int setLoc( EarthPoint ept );
      int setLoc( float rlat, float rlon );
      /** Gets the location \returns Earthpoint */
      inline EarthPoint getLoc(){ return ept; };
      /** Gets the latitude */
      inline float getLat(){ return ept.lat; };
      /** Gets the longitude */
      inline float getLon(){ return ept.lon; };
      /** Sets an index value */
      inline int setIndex( int rindex ){ index = rindex; return 1; };
      /** Gets the index value */
      inline int getIndex(){ return index; };
      int setType( int t );
      /** Gets the plot type */
      inline int getType(){ return type; };
      int setData( char str );
      int setData( const char *str );
      int clearData();
      int formatData( const char *format, ... );
      /** Returns the data */
      inline const char *getData(){ return data; };
      /** Returns the format */
      inline const char *getFormat(){ return format; };
      int setFormat( const char *str );
      int addFormat( const char *str );
      int setValue( int i, float val );
      int offsetValue( int i, float val );
      int scaleValue( int i, float val );
      int setValue( float val );
      int setValue( float val1, float val2 );
      float getValue( int i );
      int print();

      friend class DatumPlot;
      /** Plot types */
      enum Type {
         NONE,
         DATA,   /**< Data, either formatted value or text */
         VALUE,  /**< A floating point value */
         TEXT,   /**< Text string */
         MARK,   /**< Marker, text is type */
         INT,    /**< Integer value */
         CLOUD,  /**< Cloud symbol, text is type */
         WBARB,  /**< Wind barb, value[0] = dir, value[1] = spd */
         CBARB   /**< Wind barb, offset for cloud symbol, value[0] = dir, value[1] = spd */,
         DBARB   /**< Wind barb, offset for dot, value[0] = dir, value[1] = spd */,
         VECT    /**< Wind vector, value[0] = dir, value[1] = spd */,
         AVECT   /**< Wind vector, dot for no wind, value[0] = dir, value[1] = spd */,
         WTRACK  /**< Wind track, value[0] = dir, value[1] = spd */,
         SYMB,   /**< Data symbol, text is type */
         LINE,   /**< Draws a line with either previous or next Datum */
         CONST
      };
   };
}   
#endif
