#ifndef _WINC_DOMAIN
#define _WINC_DOMAIN

#include <WXP/Const.h>
#include <WXP/DomainVal.h>
#include <WXP/EarthPoint.h>
#include <WXP/GridPoint.h>
#include <WXP/Point.h>
#include <WXP/Points.h>
#include <WXP/Rect.h>
#include <WXP/String.h>

namespace WXP {
   /*
      Structure for domain location
   */
   class Domain : private DomainVal {
      Rect limits;
      Rect view;
   
      float sat_rad;
      float sat_rad2;
      float sat_rad4;
      float sat2_earth2;
      float earth_sat_ratio;
      float earth_ratio;
      float earth_ratio2;
      float tlat1;
      float tlat2;
      float cx, cy;
      float xfactor;
      float mult;
      float yfactor;
      float scale;

      public:
      static float earth_rad;  /**< Mean earth radius */
      static float earth_maj;  /**< Major axis earth radius */
      static float earth_min;  /**< Minor axis earth radius */

      Domain();
      int init();
      int set( DomainVal &val );
      int set( const char *str );
      int set( EarthPoint &ul, EarthPoint &lr, int rnx, int rny );
      int set( int rnx, int rny );
      DomainVal get();
      /** Returns true if domain is set to something */
      inline bool isSet(){ return proj != Const::MISS? true: false; };
      int setParam();
      int setLimits();
      int getLimits( Rect &rect );
      Rect getLimits();
      int setView( Rect &rect );
      /** Returns projection type */
      inline int getProj(){ return proj; };
      /** Returns projection latitude */
      inline float getPLat(){ return plat; };
      /** Returns projection longitude */
      inline float getPLon(){ return plon; };
      /** Returns central latitude */
      inline float getCLat(){ return clat; };
      /** Returns central longitude */
      inline float getCLon(){ return clon; };
      /** Returns number of gridpoints in X */
      inline int getNx(){ return nx; };
      /** Returns number of gridpoints in Y */
      inline int getNy(){ return ny; };
      /** Returns grid spacing in X */
      inline float getDx(){ return dx; };
      /** Returns grid spacing in Y */
      inline float getDy(){ return dy; };
      /** Returns projection parameters */
      inline float getParam( int i ){ return i >= 0 && i < 5 ? param[i]: Const::MISS; };
      int tranCenter( Point &pt );
      float getAspect();
      int tran( EarthPoint &ept, Point &pt );
      int tran( Point &pt, EarthPoint &ept );
      int tran( GridPoint &gpt, Point &pt );
      int tran( Point &pt, GridPoint &gpt );
      int tran( EarthPoint &ept, GridPoint &gpt );
      int tran( GridPoint &gpt, EarthPoint &ept );

      /** Translation location */
      enum TranLocate { 
	 UL, /**< Upper left point */
	 UR,     /**< Upper right point */
	 LL,     /**< Lower left point */
	 LR,     /**< Lower right point */
	 CENTER  /**< Center point */
      };
      int tran( int w, EarthPoint &ept );
      int tran( int w, GridPoint &gpt );
      int tran( int w, Point &pt );

      bool within( Point &pt );
      bool within( EarthPoint &ept );
      bool withinExt( Point &pt, int ext );
      bool withinExt( EarthPoint &ept, int ext );
      int where( Point &pt );
      int where( EarthPoint &ept );
      int scaleSize( int width, int height );
      int scaleGrid( float scale );
      int zoom( Point &pt, float zfac );
      int pan( Point &pt );
      int adjust( Point &pt );
      int adjust( Points &pts );
      float adjustDir( float dir, Point &pt );
      float adjustDir( float dir, EarthPoint &ept );
      float unadjustDir( float dir, Point &pt );
      float unadjustDir( float dir, EarthPoint &ept );
      int offset( Point &pt, float dir, float rad, Point &opt );
      float direct( Point &pt1, Point &pt2 );
      float getVert();
      float getMapFactor( EarthPoint &ept );
      int wrapLine( Point &pt1, Point &pt2 );
      int wrapLine( Points &pts );
      int offsetLine( Point &pt1, Point &pt2, int flag );
      int offsetLine( Points &pts, int flag );
      int toString( String &string );
      int print();
      int printLimits();
   
      /** Returns if projection matches p */
      inline int isProj( int p ){ return p == proj; };
      /** Is the projection a map */
      inline int isMap(){ return isMap( proj ); };
      /** Is the projection a thermo diagram */
      inline int isTherm(){ return isTherm( proj ); };
      /** Is the projection a coordinate plot */
      inline int isCoord(){ return isCoord( proj ); };
      /** Is the projection a polar plot */
      inline int isPolar(){ return isPolar( proj ); };

      static int getProj( const char *proj );
      static bool compare( Domain &dom1, Domain &dom2 );
      static bool compareProj( Domain &dom1, Domain &dom2 );
      static int translate( Domain &dom1, Domain &dom2, Point &pt1, Point &pt2 );
      static float calcEarthDist( EarthPoint &ept1, EarthPoint &ept2 );
      static int offsetLatLon( EarthPoint &ept, float dir, float rad, EarthPoint &opt );

      static const int PSTEREO = DomainVal::PSTEREO;/**< Polar stereographic */
      static const int LATLON  = DomainVal::LATLON;/**< Latitude longitude */
      static const int MERC    = DomainVal::MERC;  /**< Mercator */
      static const int LAMB    = DomainVal::LAMB;  /**< Lambert conformal */
      static const int ORTHO   = DomainVal::ORTHO; /**< Orthographic */
      static const int GNOM    = DomainVal::GNOM;  /**< Gnomic */
      static const int MOLL    = DomainVal::MOLL;  /**< Molleweide */
      static const int LAMBAZ  = DomainVal::LAMBAZ;/**< Lambert azithumal */
      static const int ALBERS  = DomainVal::ALBERS;/**< Albers equal area */
      static const int SAT     = DomainVal::SAT;   /**< Satellite */
      static const int RADIAL  = DomainVal::RADIAL;/**< Radar Radial */
      static const int PIXEL  = DomainVal::PIXEL;  /**< Pixel space */
      static const int XY     = DomainVal::XY;     /**< XY Cartesian */
      static const int XZ     = DomainVal::XZ;     /**< XZ Cartesian */
      static const int XP     = DomainVal::XP;     /**< X-Pres Cartesian */
      static const int XLOGP  = DomainVal::XLOGP;  /**< X-logP Cartesian */
      static const int POLAR  = DomainVal::POLAR;  /**< Polar coordinate */
      static const int CAT    = DomainVal::CAT;    /**< Category plot */
      static const int SKEWT  = DomainVal::SKEWT;  /**< SkewT thermodynamic diagram */
      static const int STUVE  = DomainVal::STUVE;  /**< Stuve thermodynamic diagram */
      static const int EMA    = DomainVal::EMA;    /**< Emagram thermodynamic diagram */
      static const int TEPH   = DomainVal::TEPH;   /**< Tephigram thermodynamic diagram */
      static const int RTEPH  = DomainVal::RTEPH;  /**< Rotated Tephigram thermodynamic diagram */
      static const int ZOOM   = DomainVal::ZOOM;   /**< Specifies zoom parameters */
      static const int SIZE   = DomainVal::SIZE;   /**< Specifies size of grid/image only */

      /** Is the projection a map */
      static inline bool isMap( int a ){ return (a >= PSTEREO && a <= SAT);}
      /** Is the projection a thermo diagram */
      static inline bool isTherm( int a ){ return (a >= SKEWT && a <= RTEPH);}
      /** Is the projection a coordinate plot */
      static inline bool isCoord( int a ){ return (a >= PIXEL && a <= CAT);}
      /** Is the projection a polar plot */
      static inline bool isPolar( int a ){ return (a == POLAR);}

      static float OFFSET_LL;   /**< Projection limit for Lat/Lon */
      static float OFFSET_MERC; /**< Projection limit for Mercator */
   };
}
#endif
