#ifndef _WINC_EARTHPOINTS
#define _WINC_EARTHPOINTS

#include <WXP/Const.h>
#include <WXP/EarthPoint.h>
#include <WXP/EarthRange.h>

namespace WXP {
   class EarthPoints {                /* Coordinate data structure */
      int num;
      int max;
      int ind;
      float *lat;     /**< Latitude value */
      float *lon;     /**< Longitude value */
   
      public:
      int init();
      int init( int size );
      EarthPoints();
      EarthPoints(int size);
      ~EarthPoints();
      EarthPoints( const EarthPoints &data );
      EarthPoints operator=( const EarthPoints &data );
      int copy( const EarthPoints &data );
      /** Clears the array */
      inline int clear(){ num = 0; return 1; };
      int expand( int size );
      int add( float lat, float lon );
      int add( EarthPoint ept );
      /** Simple add, no bounds check */
      inline int add1( EarthPoint ept ){ 
	 lat[num] = ept.lat; lon[num] = ept.lon; num++; return 1; };
      int add( int ind );
      /** Simple add, no bounds check */
      inline int add1( int ind ){ 
	 lat[num] = lat[ind]; lon[num] = lon[ind]; num++; return 1; };
      int set( int ind, float rlat, float rlon );
      int set( int ind, EarthPoint &ept );
      int get( int ind, EarthPoint &ept );
      EarthPoint get( int ind );
      /** Gets the latitude of a point */
      inline float getLat( int ind ){ 
	 return ind >= num || ind < 0 ? Const::MISS : lat[ind]; };
      /** Gets the longitude of a point */
      inline float getLon( int ind ){ 
	 return ind >= num || ind < 0 ? Const::MISS : lon[ind]; };
      /** Gets the latitude of a point, no bounds check */
      inline float getLat1( int ind ){ return lat[ind]; };
      /** Gets the longitude of a point, no bounds check */
      inline float getLon1( int ind ){ return lon[ind]; };
      /** Return the number of points */
      inline int getNum() { return num; };
      int offset( int ind, float rlat, float rlon );
      int within( EarthPoint &ept );
      EarthRange getRange();
      int print();
   };
}
#endif
