#ifndef _WINC_FILE
#define _WINC_FILE

#include <stdio.h>
#include <WXP/Config.h>
#ifdef USE_CURL
#include <curl/curl.h>
#endif

#include <WXP/Buffer.h>
#include <WXP/FileAttr.h>
#include <WXP/String.h>

namespace WXP {
   class File : public FileAttr {
      int type;
      int stype;
      FILE *fp;
#ifdef USE_CURL
      CURL *easy;
      CURLM *multi;
#endif
      int fd;
      void *ptr;
      bool opened;
      int access;
      int shared;
      Buffer buf;
   
#ifdef USE_CURL
      static size_t curl_write( char *buffer, size_t size, size_t nitems, 
	    void *userp);
      static size_t curl_header( char *buffer, size_t size, size_t nitems, 
	    void *userp);
      int curl_fill_buffer( int size );
#endif

      public:
      int init( );
      File( const char *file, const char *raccess );
      File( String &file, const char *raccess );
      File( const char *file, int raccess );
      File();
      ~File();
      int open( const char *file, const char *raccess );
      int open( String &file, const char *raccess );
      int open( const char *file, int raccess );
      int getLine( char *string, int len );
      int getLine( String &string );
      int skipLine();
      bool isOpen();
      int rewind();
      int close();
      bool endFile();
      int scanf( const char *fmt, ... );
      int printf( const char *fmt, ... );
      int read( unsigned char *buf, int size, int num );
      int read( unsigned char *buf, int size );
      int read( Buffer &buf, int size );
      int read( Buffer &buf );
      int write( const unsigned char *buf, int size, int num );
      int write( const unsigned char *buf, int size );
      int tell();
      int seek( int byte, int where );
      /** Seeks to byte position in file offset from beginning */
      inline int seek( int byte ){ return seek( byte, 0 ); };
      int setShare( int mode );
      FILE *getFP();
   
      enum Type {
         REG,
         PIPE,
         STDIO,
         FIFO,
         DEVICE,
         SOCKETC,
         SOCKETS,
         HTTP,
         FTP };
   
      static const int FILE_LENGTH = 100;
      /*
         define file type
      */
      enum Format {
         UNK,
         WXP,
         NETCDF,
         GRIB,
         BUFR,
         UNISYS,
         NONE };
   
   };
}
#endif
