#ifndef _WINC_FILENAME
#define _WINC_FILENAME

#include <WXP/Date.h>
#include <WXP/DateVal.h>
#include <WXP/NameConv.h>
#include <WXP/String.h>
#include <WXP/Strings.h>

namespace WXP {
   class Filename {
      bool is_set;
      bool fixed;
      char type[30];
      String name; 
      String hdr; 
      char param[50];
      Date date;        // Actual date of file
      Date base_date;   // Base date for latest data (time of latest file)
                        // used for multiple files
      NameConv conv;
      Strings files;
      int num_file;
      int num_off;
      int num_type;
      int num_dir;
      int file_off;
   
      public:
      int initParam();
      int init();
   
      Filename( const char *nameconv, const char *str );
      Filename( const char *nameconv, Date &rdate );
      Filename( const char *filename );
      Filename();
      ~Filename();
   
      int set( const char *nameconv, const char *str );
      int set( const char *nameconv, Date &rdate );
      int set( const char *filename );
      bool isSet();

      int setNameConv( const char *nameconv );
      int setPath( const char *path );
   
      int setDate( Date &rdate );
      int setDate( Date &rdate, const char *param );
      int getDate( Date &rdate );
      DateVal getDateval();
      int getOffset();
   
      int setFixDate( DateVal &rdate );
      int setFixDate( const char *str );

      int setType( const char *rtype );
      const char *getType();

      inline int setForeTime( int time ){ conv.setForeTime( time ); return 1; };
      inline int getForeTime(){ return conv.getForeTime(); };

      inline int setKey( char ch, const char *str ){ return conv.setKey(ch, str); };
      inline const char *getExt(){ return conv.getExt(); };
      inline const char *getKey( char ch ){ return conv.getKey(ch); };
   
      int setName( const char *rname );
      const char *getName();
      const char *getName( int ind );
   
      int setHeader( const char *rname );
      const char *getHeader();
      bool validHeader();
   
      int addOutPath( const char *path );

      bool isFixed();

      int setNumFiles( int val );
      int setNumFiles( const char *str );
      int getNumFiles();
      inline int getNumType(){ return num_type; };
   
      int setOutput( bool val );
      int setLateLimit( int val );
      int setLateDate( Date rdate );
      int setBaseDate( Date &rdate );
   
      int checkFile( const char *file );
      int setFile( const char *file );
      int setFilePrompt( const char *prompt );
      bool exist();
      int setFile( int ind );
   
      int listFile();
      int print();
      int printConv();
   
      static const int UNK = 0;   /**< Multiple files by time */
      static const int HOURS = 1;   /**< Multiple files by time */
      static const int FILES = 2;   /**< Multiple files by list of files */
   };
}   
#endif
