#ifndef _WINC_FORETIME
#define _WINC_FORETIME

#include <WXP/Const.h>
#include <WXP/DateVal.h>
#include <WXP/String.h>

namespace WXP {
   class ForeTime {
      int type;
      int unit;
      float tim1, tim2;
      float off;
      DateVal end;
   
      public:
      ForeTime();
      ForeTime( int t, int t1, int t2 );
      ForeTime( int t );
      ForeTime( const char *str );
      int init();
      /** Returns if object values set */
      inline bool isSet(){ return type != NONE; };
      int decode( const char *str );
      int toString( String &str );
      float getFore();
      float getHours();
      int getSeconds();
      int toTimeLabelDay( String &str );
      static int toTimeLabel( float ftime, String &str );
      int toTimeLabel( String &str );
      int toVarLabel( String &str );
      int set( int t, float t1, float t2 );
      int set( int t, int u, float t1, float t2 );
      int set( int val );
      int set( const char *str );
      int setEnd( DateVal &dv );
      int setOffset( int val ){ off = val; return 1; };
      float getOffset(){ return off; };
      inline int getType(){ return type; };
      inline bool isTypeComplex(){ return type > 0 && type < 9; };
      inline int setTime1( float val ){ tim1 = val; return 1; };
      inline float getTime1(){ return tim1; };
      inline int setTime2( float val ){ tim2 = val; return 1; };
      inline float getTime2(){ return tim2; };
      inline DateVal getEnd(){ return end; };
      inline bool validEnd(){ return end.year != Const::MISS; };
      inline bool valid(){ return tim1 != Const::MISS || tim2 != Const::MISS; };
      int toCode();
      int print();
      /*
         Time types
      */
      static const int NONE =  -9999;
      static const int FORE =      0;  /**< Single forecast time */
      static const int ANAL =      1;  /**< Analysis */
      static const int RNG  =      2;  /**< Forecast time range */
      static const int AVG  =      3;  /**< Average over forecast time range */
      static const int ACC  =      4;  /**< Accumulation over forecast time range */
      static const int DIFF =      5;  /**< Difference between two forecast times */
      static const int MAX  =      6;  /**< Maximum over a period */
      static const int MIN  =      7;  /**< Minimum over a period */
      static const int ANY  =      9;  /**< Match any type */
      /*
       * Time units
       */
      static const int MINUTE = 0;  /**< Time in Minutes */
      static const int HOUR = 1;  /**< Time in Hours */
      static const int DAY  = 2;  /**< Time in Days */
      static const int MONTH = 3;  /**< Time in Months */
      static const int YEAR = 4;  /**< Time in Years */
   };
}
#endif
