#ifndef _WINC_GRID
#define _WINC_GRID

#include <WXP/Date.h>
#include <WXP/DateVal.h>
#include <WXP/Domain.h>
#include <WXP/DomainVal.h>
#include <WXP/EarthPoint.h>
#include <WXP/ForeTime.h>
#include <WXP/GridPoint.h>
#include <WXP/GridSpec.h>
#include <WXP/Level.h>
#include <WXP/Point.h>
#include <WXP/Range.h>
#include <WXP/VarSpec.h>
#include <WXP/Variable.h>

namespace WXP {
   class Grid {
      int type;                 /* Grid type used in grid */
      int version;              /* Version of grid */
      int source;               /* Source of grid */
      char datim[50];           /* Date and time information */
      char info[90];            /* Label for grid information */
      Date date;                /* Initial date of file */
      int nx, ny;               /* Size of grid */
      int grid_type;            /* Grid type number */
      Domain domain;            /* The grid location */
      int model;                /* Model number */
      char gen_name[20];        /* Model used to generate grid */
      VarSpec var;
      float *data;              /* A pointer to the grid */
      float *xloc;              /* A pointer to X location of grid */
      float *yloc;              /* A pointer to Y location of grid */

      int initClass();
   
      public:
      Grid();
      Grid( int rnx, int rny );
      Grid( const Grid &grid );
      ~Grid();
      int init();
      int copy( const Grid &grid );
      int copyInfo( const Grid &grid );
      int copyData( const Grid &grid );
      Grid operator=(const Grid &grid);
      int setType( int val );
      int getType();
      int setSource( int val );
      int getSource();
      int setVersion( int val );
      int getVersion();
      int setDatim( const char *str );
      const char *getDatim();
      int setInfo( const char *str );
      const char *getInfo();
      int setDate( Date &d );
      int setDate( DateVal &dv );
      inline Date getDate(){ return date; };
      inline Date getDate( Date &rdate ){ rdate = date; return 1; };
      Date getValidDate();
      DateVal getDateVal();
      int setSize( int x, int y );
      int getSize( int &x, int &y );
      int getNx();
      int getNy();
      float getDx();
      float getDy();
      inline int setGridType( int type ){ grid_type = type; return 1; };
      inline int getGridType(){ return grid_type; };
      int setDomain( DomainVal &dv );
      int setDomain( Domain &dom );
      DomainVal getDomainVal();
      Domain getDomain();
      int getDomain( Domain &dom );
      int getProj();
      int tran( EarthPoint ept, Point &pt );
      int tran( Point pt, EarthPoint &ept );
      int tran( GridPoint gpt, Point &pt );
      int tran( Point pt, GridPoint &gpt );
      int tran( EarthPoint ept, GridPoint &gpt );
      int tran( GridPoint gpt, EarthPoint &ept );
      int tran( Domain &dom, Point pt1, Point &pt2 );
      int tran( Point pt1, Domain &dom, Point &pt2 );
      float adjustDir( float dir, Point &pt );
      float unadjustDir( float dir, Point &pt );
      inline int setModel( int val ){ model = val; return 1; };
      inline int getModel(){ return model; };
      int setGenName( const char *str );
      inline const char *getGenName(){ return gen_name; };
      inline int setVar( VarSpec v ){ var = v; return 1; };
      inline int getVar( VarSpec &v ){ v = var; return 1; };
      inline VarSpec getVarSpec(){ return var; };
      int setLevel( const char *str );
      int setLevel( Level &l );
      int getLevel( Level &l );
      Level getLevel();
      int getLevelType();
      int setForeTime( const char *str );
      int setForeTime( ForeTime &t );
      int getForeTime( ForeTime &t );
      ForeTime getForeTime();
      float getForeHours();
      int getForeSeconds();
      int setVar( const char *str );
      int setVar( Variable &v );
      const char *getUnits();
      int setUnits( const char *units );
      Variable getVar();
      GridSpec getSpec();
      int getSpec(GridSpec &spec);
      int allocData();
      int allocLocate();
      int setValue( int x, int y, float val );
      float getValue( int x, int y );
      inline float getValue1( int x, int y ){ return data[y*nx+x]; };
      float getValue( GridPoint gpt );
      float getValue( EarthPoint ept );
      float getValue( Point pt );
      bool isLocate();
      bool isData();
      bool isValue();
      int setLocate( int x, int y, Point &pt );
      int getLocate( int x, int y, Point &pt );
      Point getLocate( int x, int y );
      int getValid();
      float getAverage();
      int getRange( float &min, float &max );
      int getRange( Range &range );
      int scale( float scale );
      int offset( float offset );
      int print();
      int printInfo();
      int printStats();
      int printData();

      static int copyInfo( Grid &grid1, const Grid &grid2 );
      static int copyData( Grid &grid1, const Grid &grid2 );
      static int copy( Grid &grid1, const Grid &grid2 );
   };
}   
#endif
