#ifndef _WINC_GRIDFILE
#define _WINC_GRIDFILE

#include <WXP/Filename.h>
#include <WXP/GribFile.h>
#include <WXP/Grid.h>
#include <WXP/GridWxpFile.h>

namespace WXP {
   class GridFile {
      int type;               /* The type of grid file */
      int subtype;            /* The subtype of grid file */
      int access;
      bool opened;
      int use_header;
      GridWxpFile wxp;
      GribFile grib;

      public:
      enum Type {
         UNK,
         WXP,
         WXPB,
         NETCDF,
         GRIB };
   
      GridFile();
      GridFile( const char *filename, int access, int type);
      GridFile( const char *filename );
      GridFile( Filename &filename, int access, int type);
      GridFile( Filename &filename );
      ~GridFile();
   
      int initClass();
      int init();
      int setVer( int v );
      int setParam( const char *str );
      int useHdrFile( bool val );
      int setMaxBad( int val );
      int setMissValue( float rmiss );
      int setValueBits( int val );
      int setFormat( const char *rform );
      int open( const char *filename, int access, int type);
      int open( const char *filename );
      int open( Filename &filename, int access, int type);
      int open( Filename &filename );
      bool isOpen();
      int getType();
      int close();
      int rewind();
      int peek( Grid &grid );
      int read( Grid &grid );
      int read( const char *filename, int type, Grid &grid );
      int read( const char *filename, Grid &grid );
      int readHdr( Grid &grid );
      int readData( Grid &grid );
      int read( Grid &grid, const char *spec );
      int read( GridSpec &spec, Grid &grid );
      int read( Grids &grids );
      int write( Grid &grid );
      int write( const char *filename, int type, Grid &grid );
      int write( const char *filename, Grid &grid );
   
      static int checkType( const char *filename );
   };
}   
#endif
