#ifndef _WINC_GRIDSPEC
#define _WINC_GRIDSPEC

#include <WXP/Const.h>

namespace WXP {
   class GridSpec {
      int num;                  /* Specific grid number */
      int model;                /* Model used to produce grid */
      int type;                 /* Grid type used in grid */
      int time;                 /* Time of grid */
      int ltype;                /* Type of vertical level */
      int level;                /* Level of grid */
      int var[5];               /* Variable */
      char units[20];           /* Variable units */

      public:
      GridSpec();
      GridSpec( const char *str );
      int init();
      int set( int rmod, int rtype, int t, int lt, int l, int v );
      int set( const char *str );
      int decode( const char *str );
      inline int setNum( int val ){ num = val; return 1; };
      inline int getNum(){ return num; };
      inline int setModel( int val ){ model = val; return 1; };
      inline int getModel(){ return model; };
      inline int setType( int val ){ type = val; return 1; };
      inline int getType(){ return type; };
      inline int setTime( int val ){ time = val; return 1; };
      inline int getTime(){ return time; };
      inline int getTimeType(){ return (time/1000000)%10; };
      inline int getTimeOffset(){ return time/10000000; };
      inline int getTime1(){ return time%1000; };
      inline int getTime2(){ return (time/1000)%1000; };
      inline int setLevelType( int val ){ ltype = val; return 1; };
      inline int getLevelType(){ return ltype; };
      inline int setLevel( int val ){ level = val; return 1; };
      inline int getLevel(){ return level; };
      inline int setVar( int val ){ var[0] = val; return 1; };
      int setVar( int i, int val );
      inline int getVar(){ return var[0]; };
      bool isSet();
      bool isVarSet();
      int setUnits( const char *str );
      inline const char *getUnits(){ return units; };
      int print();

      static bool equal( GridSpec gs1, GridSpec gs2 );
   };
}
#endif
