#ifndef _WINC_LZWCOMP
#define _WINC_LZWCOMP

#include <WXP/File.h>
#include <WXP/Image.h>

namespace WXP {
#define MAX_COLOR 256
   
   class LZWComp {
      File file;              /* The file */
      int access;
      bool opened;
      unsigned char data[1000]; /* Data buffer */
      unsigned char *ptr;     /* Pointer to data array */
      int width;              /* Width of image */
      int height;             /* Height of image */
      int depth;              /* Depth of image */
      int use_inter;
      int trans;
      /*
         Read parameters 
      */
      int glob_color_exist;   /* Specifies whether global color table exists */
      int glob_color_size;    /* Specifies global color table size */
      int local_color_exist;  /* Specifies whether local color table exists */
      int local_color_size;   /* Specifies local color table size */
   
      int data_size;          /* Data array size */
      int data_ind;           /* Index into data array */
      /* 
         Data codes
      */
      int code_size;          /* Initial coded data bit size */
      int clear_code;         /* Clear code */
      int end_info_code;      /* End of information code */
      int coded_data;         /* Current coded information */
      int coded_prev;         /* Previous piece of coded data */
      int coded_mask;         /* Coded data mask used to read in data */
      int coded_size;         /* Current coded data bit size */
      /*
         String table
      */
      int tab_pfix[4096];
      int tab_ext[4096];
      int tab_ind;            /* Pointer into the string table */
      /*
         Pixel stack
      */ 
      int pix_stack[5000];    /* The pixel stack */
      int pix_ptr;            /* Pointer into the pixel stack */
   
      unsigned int data_bits; /* Number bits encoded into current byte */
      unsigned int temp_data; /* Temporary storage for current byte */
     
      /*
         Encode hash table
      */
      struct HashNode {        
   	 int hash;
   	 int val;
   	 HashNode *next;
      };

      HashNode *htable[4096];
      int hnodes;
    
      unsigned int cur_data;  /* Current pixel,encoded data */
      unsigned int prev_data; /* Previous encoded data */
      int out_bytes;          /* Number of bytes written to output file */
      int num_pixel;          /* Number of pixels read in from file */

      int file_num;
      int loop_end_delay;  /* Delay */
      int loop_delay;  /* Delay */

      public:
      LZWComp();
      ~LZWComp();
      int init();
   
      LZWComp( const char *filename, int access );
      int open( const char *filename, int access );
      bool isOpen();
      int close();
      int setInterlace( int val );
      /*
         Decode routines
      */
      int read( Image &image );
      private:
      int readData();
      int pushPix( int val );
      int popPix();
      /*
         Encode routines
      */
      public:
      int write( Image &image );
      int write( const char *file );
      private:
      int outCode( unsigned int code );
      int clearTable();
   };
}      
#endif
