#ifndef _WINC_LEVEL
#define _WINC_LEVEL

#include <WXP/String.h>
#include <WXP/VCoord.h>

namespace WXP {
   class Level {
      struct Lev {
	 int type;
	 float lev;
      };
      Lev l1;   // Lower level
      Lev l2;   // Upper level
   
      int decodeLev( const char *str, Lev &l );

      public:
      Level();
      Level( int t1, float l1, int t2, float l2 );
      Level( int t, float l1, float l2 );
      Level( int t, int l );
      Level( const char *str );
      int init();
      /** Is level set */
      inline bool isSet(){ return l1.type != NONE; }; 
      int decode( const char *str );
      int toTypeCode();
      int toLevelCode();
      int set( int t1, float l1, int t2, float l2 );
      int set( int t, float l1, float l2 );
      int set( int t, float l );
      int decode( int t, int l );
      int set( const char *str );
      /** Get level type */
      inline int getType(){ return l1.type; }; 
      /** Get lower level type */
      inline int getType1(){ return l1.type; }; 
      /** Get upper level type */
      inline int getType2(){ return l2.type; }; 
      /** Get lower level value */
      inline float getLev1(){ return l1.lev; }; 
      /** Get upper level value */
      inline float getLev2(){ return l2.lev; }; 
      int getLower( VCoord &vc );
      int getUpper( VCoord &vc );
      int toString( String &str );
      int toLabel( String &str );
      /** Is level valid */
      inline bool valid(){ return l1.lev != Const::MISS; }; 
      int print();
      bool isLayer();
      bool isPres();
      bool isHght();
      bool isPot();

      static int toTypeCode( int type1, int type2 );
      static int toTypeCode( int type1 );
      /*
         Pressure level types
      */
      static const int NONE    = -9999;  /**< No level */
      static const int SND         = 0;  /**< Whole sounding */
      static const int SFC         = 1;  /**< Surface */
      static const int CLD_BOT     = 2;  /**< Cloud bottom */
      static const int CLD_TOP     = 3;  /**< Cloud top */
      static const int FRZ         = 4;  /**< Freezing level */
      static const int LCL         = 5;  /**< LCL level */
      static const int MWND        = 6;  /**< Maximum wind level */
      static const int TROP        = 7;  /**< Tropopause */
      static const int TOP         = 8;  /**< Top of atmosphere */
      static const int SEABOT      = 9;  /**< Sea bottom */
      static const int WHOLE      = 10;  /**< Whole atmosphere */
      static const int CB_BASE    = 11;  /**< Cumulnumbus base */
      static const int CB_TOP     = 12;  /**< Cumulnumbus top */
      static const int PR        = 100;  /**< Pressure level */
      static const int MSL       = 101;  /**< Mean sea level */
      static const int HT        = 102;  /**< Height level */
      static const int HAG       = 103;  /**< Height above ground level */
      static const int SIG       = 104;  /**< Sigma level */
      static const int HYB       = 105;  /**< Hybrid level */
      static const int HBG       = 106;  /**< Height below ground level */
      static const int POT       = 107;  /**< Potential temperature level */
      static const int PAG       = 108;  /**< Pressure above ground level */
      static const int POT_VORT  = 109;  /**< Potential vorticty level */
      static const int ETA       = 111;  /**< Eta level */
      static const int MISC      = 199;  /**< Misc level */
      static const int WHOLEL    = 200;  /**< Whole atmosphere layer */
      static const int TOP_FRZ   = 204;  /**< Top freezing level */
      static const int GSCLD_BOT = 206;  /**< Grid scale layer cloud bottom */
      static const int GSCLD_TOP = 207;  /**< Grid scale layer cloud top */
      static const int BLCLD_BOT = 209;  /**< Boundary layer cloud bottom */
      static const int BLCLD_TOP = 210;  /**< Boundary layer cloud top */
      static const int BLCLD_LYR = 211;  /**< Boundary layer cloud layer */
      static const int LCLD_BOT  = 212;  /**< Low cloud bottom */
      static const int LCLD_TOP  = 213;  /**< Low cloud top */
      static const int LCLD_LYR  = 214;  /**< Low cloud layer */
      static const int CLD_CEIL  = 215;  /**< Cloud ceiling */
      static const int PBL       = 220;  /**< Planetary boundary layer */
      static const int MCLD_BOT  = 222;  /**< Middle cloud bottom */
      static const int MCLD_TOP  = 223;  /**< Middle cloud top */
      static const int MCLD_LYR  = 224;  /**< Middle cloud layer */
      static const int HCLD_BOT  = 232;  /**< High cloud bottom */
      static const int HCLD_TOP  = 233;  /**< High cloud top */
      static const int HCLD_LYR  = 234;  /**< High cloud layer */
      static const int CONV_BOT  = 242;  /**< Convective cloud bottom */
      static const int CONV_TOP  = 243;  /**< Convective cloud top */
      static const int CONV_LYR  = 244;  /**< Convective cloud layer */
      static const int WBZERO    = 245;  /**< Wet bulb zero */
      static const int MAXEPOT   = 246;  /**< Maximum equiv potential temp level */
      static const int EQUILIB   = 247;  /**< Equilibrium level */
      static const int SCCLD_BOT = 248;  /**< Shallow convect cloud bottom */
      static const int SCCLD_TOP = 249;  /**< Shallow convect cloud top */
      static const int DCCLD_BOT = 251;  /**< Deep convect cloud bottom */
      static const int DCCLD_TOP = 252;  /**< Deep convect cloud top */
      static const int SCLW_BOT  = 253;  /**< Supercool liquid water bottom */
      static const int SCLW_TOP  = 254;  /**< Supercool liquid water top */
   };
}
#endif
