#ifndef _WINC_MDRDATA
#define _WINC_MDRDATA

#include <WXP/Const.h>
#include <WXP/Date.h>
#include <WXP/EarthPoint.h>
#include <WXP/MdrRegion.h>

namespace WXP {

   class MdrData {
      char id[5];
      char region[11];      /* Region to which the station is located */
      float lat,lon;
      float elev;
      Date date;

      int status;

      MdrRegion reg[10];
      int num_reg;

      char echo[26][26];

      public:
      MdrData();
      int init();
      inline const char *getId(){ return id; };
      int setLoc( float rlat, float rlon );
      inline Date getDate(){ return date; };
      inline int getStatus(){ return status; };
      inline int getPrecipCover( int ind, int pind ){ return reg[ind].prec[pind].cover; };
      inline const char *getPrecipType( int ind, int pind ){ return reg[ind].prec[pind].type; };
      inline int getPrecipIntensity( int ind, int pind ){ return reg[ind].prec[pind].intens; };
      inline int getNumPrecip( int ind ){ return reg[ind].num_prec; };
      inline float getLocDir( int ind, int pind ){ return reg[ind].loc[pind].dir; };
      inline float getLocRad( int ind, int pind ){ return reg[ind].loc[pind].rad; };
      inline int getNumLoc( int ind ){ return reg[ind].num_loc; };
      int getRegLoc( int ind, EarthPoint &ept );
      inline bool validTops( int ind ){ return reg[ind].tops.dir != Const::MISS ? true : false; };
      inline float getTopsDir( int ind ){ return reg[ind].tops.dir; };
      inline float getTopsRad( int ind ){ return reg[ind].tops.rad; };
      inline float getTopsHgt( int ind ){ return reg[ind].tops.hgt; };
      int getTopsLoc( int ind, EarthPoint &ept );
      inline char getMoveType( int ind, int pind ){ return reg[ind].move[pind].type; };
      inline float getMoveDir( int ind, int pind ){ return reg[ind].move[pind].dir; };
      inline float getMoveSpd( int ind, int pind ){ return reg[ind].move[pind].spd; };
      inline int getNumMove( int ind ){ return reg[ind].num_move; };
      inline int getNumReg(){ return num_reg; };

      int print();
      int printEchoes();

      enum Status {
	 NA,
	 NE,
	 OM,
	 PR
      };
   
      friend class MdrFile;
      friend class MdrWxpFile;
      friend class MdrImage;
      friend class MdrPlot;
      friend class MdrDecode;
   };
}   
#endif
