#ifndef _WINC_MDRPLOT
#define _WINC_MDRPLOT

#include <WXP/CityHash.h>
#include <WXP/Date.h>
#include <WXP/DatumPlot.h>
#include <WXP/Domain.h>
#include <WXP/Drawing.h>
#include <WXP/Filename.h>
#include <WXP/Grid.h>
#include <WXP/GridPlot.h>
#include <WXP/Image.h>
#include <WXP/ImagePlot.h>
#include <WXP/Lev2Plot.h>
#include <WXP/Listener.h>
#include <WXP/MdrDomain.h>
#include <WXP/MdrData.h>
#include <WXP/Plot.h>
#include <WXP/String.h>
#include <WXP/Timezone.h>
#include <WXP/VarList.h>

namespace WXP {
   class MdrPlot : public Drawing, public Listener {
      Filename filename;
      int type;
      int fill;
      int sync;
      bool draw_plot;
      bool draw_data;
      String color_table;
      String color_fill;
      String color;
      String variable;
      char id[5];
      MdrDomain mdom;
      DatumPlot dplot;
      GridCont gcont;
      ImagePlot img;
      int plt_site;
      float plt_dist[5];
      int plt_echo;
      char cutoff[20];
      int ramp_cut;
      int ramp_len;
      int clutter;
      Timezone tz;
      CityHash chash;
      VarList vars;
      int num;
   
      public:
      MdrPlot();
      MdrPlot( const char *str );
      int init();
      int setParam( const char *str );
      int setPath( const char *path );
      int setFile();
      int setInput( const char *rtype );
      int getType( const char *rtype );
      int setNameConv( const char *name );
      int setHour( const char *str );
      int setFilename();
      int setFilename( const char *name );
      int setFilename( Date &date );
      int setFilename( Date &date, const char *param );
      inline int setForeTime( const char *fore ){ return 1; };
      int setColorTable( const char *str );
      int setColor( const char *str );
      int setColorFill( const char *str );
      int setVariable( const char *str );
      int setId( const char *str );
      int setMap( bool val );
      int getDate( Date &date );
      int getInitDate( Date &date );
      int getDateOffset();
      int getLoopType();
      int getNumFrames();
      int getDomain( Domain &domain );
      inline int getSync(){ return sync; };
      int draw( Plot &plot, Grid &grid ); 
      int draw( Plot &plot, Image &image ); 
      int draw( Plot &plot ); 
      int toImage( MdrData &rdata, Image &image );
      int drawData( Plot &plot, MdrData &rdata );
      void callback( const char *ev_str, const char *ev_data, const void *data );
      int print();

      enum Type {
	 SITE,
	 MOS,
	 MOSG
      };
   };
}
#endif
