#ifndef _WINC_METEOPLOT
#define _WINC_METEOPLOT

#include <WXP/Date.h>
#include <WXP/Drawing.h> 
#include <WXP/Filename.h> 
#include <WXP/Listener.h>
#include <WXP/Plot.h>
#include <WXP/Range.h>
#include <WXP/Rect.h>
#include <WXP/SfcData.h>
#include <WXP/String.h>
#include <WXP/VarSpec.h>

namespace WXP {
   class MeteoPlot : public Drawing, public Listener {
      int type;
      int sync;
      Filename filename; 
      char model[10];
      char mlabel[20];
      String variable; 
      PlotAttr gattr;
      PlotAttr dattr;
      PlotAttr lattr;
      Date sdate, edate;
      Timezone tz;

      int num;
      float size;
      float offset;
      float ymax, ymin;
      float inc;
      Rect view;
      Rect pview;
      int ptype;
      int hours[50];

      enum PType {
	 NORMAL,
	 CPLOT
      };

      public:
      MeteoPlot();
      MeteoPlot( const char *param );
      int init();
      int setType( int rtype );
      int setType( const char *str );
      int getType();
      int setParam( const char *str );
      int setFile();
      int setInput( const char *type );
      int setNameConv( const char *name );
      int setHour( const char *str );
      int setFilename();
      int setFilename( const char *name );
      int setFilename( Date &date );
      int setFilename( Date &date, const char *param );
      inline int setForeTime( const char *fore ){ return 1; };
      int setModel( const char *str );
      int setAttr( const char *str );
      int setGridAttr( const char *str );
      int setLabelAttr( const char *str );
      int promptId();
      int getDate( Date &date );
      int getInitDate( Date &date );
      int getDateOffset();
      int getLoopType();
      int getNumFrames();
      int getDomain( Domain &domain );
      inline int getSync(){ return sync; };
      int getSfcStation( const char *id, SfcData sdata[], int &num );
      int getMosStation( const char *id, SfcData sdata[], int &num );
      int getRange( SfcData data[], int num, VarSpec &spec, Range &range );
      float getX( float x );
      float getY( float y );
      int setPlotSize( VarSpec &spec, int type );
      int nextPlot();
      int setPlot( VarSpec &spec, Range &range );
      int drawGrid( Plot &plot, const char *label );
      int drawData( Plot &plot, SfcData data[], int num, VarSpec &spec );
      int drawPlot( Plot &plot, SfcData data[], int num, VarSpec &spec );
      int draw( Plot &plot );
      int draw( Plot &plot, SfcData data[], int rnum );
      void callback( const char *ev_str, const char *ev_data, const void *data );
      int print();

      enum Type {
	 PLOT,
	 DATA
      };
      enum {
	 SFC,
	 MOS
      };
   };
}
#endif
