#ifndef _WINC_MOSFILE
#define _WINC_MOSFILE

#include <WXP/Date.h>
#include <WXP/Filename.h>
#include <WXP/MosData.h>
#include <WXP/WmoParse.h>

namespace WXP {
   class MosFile {
      int type;
      int model;
      WmoParse parse;
      Date date;
      bool opened;
   
      public:
      MosFile();
      MosFile( const char *filename, int type );
      MosFile( const char *filename );
      MosFile( Filename &filename, int type );
      MosFile( Filename &filename );
      ~MosFile();
      int init();
      int setModel( int mod );
      int setModel( const char *str );
      const char*getModel();
      int open( const char *filename, int type );
      int open( const char *filename );
      int open( Filename &filename, int type );
      int open( Filename &filename );
      int rewind( );
      int close( );
      /** Returns true if file open */
      inline bool isOpen(){ return opened; };
      int getDate( Date &rdate );
      int read( MosData &data );
      int decodeNGM( const char *str, MosData &data );
      int decodeETA( const char *str, MosData &data );
      int decodeNAM( const char *str, MosData &data );
      int decodeAVN( const char *str, MosData &data );
      int decodeGFS( const char *str, MosData &data );
      int decodeMRF( const char *str, MosData &data );
      int decodeGFSX( const char *str, MosData &data );
      int decodeLAMP( const char *str, MosData &data );
      /** Returns the latest WMO header */
      inline const char *getWmo(){ return parse.getHeader(); };
   
      static const char*getModel( int mod );

      /** MOS types */
      enum Model { 
	 NONE,
	 NGM,   /**< NGM MOS Data */
	 ETA,   /**< Eta MOS Data */
	 NAM,   /**< NAM MOS Data */
	 AVN,   /**< Avation MOS Data */
	 GFS,   /**< GFS MOS Data */
	 MRF,   /**< MRF MOS Data */
	 GFSX,  /**< GFSX MOS Data */
	 LAMP,  /**< LAMP MOS Data */
	 PFM    /**< Point Forecast Matrix */
      };
      enum Type { 
	 UNK, 
	 WMO,   /**< WMO text data */
	 BUFR,  /**< BUFR data */
	 CDF    /**< netCDF data */
      };
   };
} 
#endif
