#ifndef _WINC_MOSUDATA
#define _WINC_MOSUDATA

#include <WXP/Date.h>
#include <WXP/EarthPoint.h>
#include <WXP/String.h>

namespace WXP {
   class MosUData {
      public:
      const static int NumData = 13;
      private:
      char id[7];               /* Station identifier */
      float lat;                /* Latitude */
      float lon;                /* Longitude */
      float elev;               /* Elevation */
      Date date;                /* Initial time of data */
   
      int model;                /* The model used in the data */
      int num;                  /* The number of forecasts */
      int hour[NumData];        /* The hour offset */
      float t[NumData][4];      /* The temperature */
      float rhum[NumData][4];     /* The relative humidity */
      float vert[NumData];      /* The 700 mb vert velocity */
      float lift[NumData];        /* The lifted index */
      float thick[NumData];      /* The 1000-500 mb thickness */
      float dir[NumData];       /* The wind direction */
      float spd[NumData];       /* The wind speed */
      float psl[NumData];       /* The sea level pressure */
      float prec[NumData];      /* The quantitative precipitation */

      public:
      enum Model { NONE, NGM, ETA, NAM };
   
      MosUData();
      int init();
      int setId( const char *rid );
      int setId( const char *rid, float rlat, float rlon );
      const char *getId();
      int getLoc( float &rlat, float &rlon );
      int getLoc( EarthPoint &ept );
      EarthPoint getLoc();
      inline float getLat(){ return lat; };
      inline float getLon(){ return lon; };
      int setElev( float relev );
      float getElev();
      int getSeconds();
   
      int setDate( Date &rdate );
      int getDate( Date &rdate );
      Date getDate();
      int setModel( int val );
      int getModel();
      int setNum( int val );
      int getNum();
      int setHour( int ind, int val );
      int getHour( int ind );
      int getInd( float hr );
      int setT( int ind, int type, float val );
      float getT( int ind, int type );
      int setRHum( int ind, int type, float val );
      float getRHum( int ind, int type );
      int setVert( int ind, float val );
      float getVert( int ind );
      int setLift( int ind, float val );
      float getLift( int ind );
      int setThick( int ind, float val );
      float getThick( int ind );
      int setDir( int ind, float val );
      float getDir( int ind );
      int setSpd( int ind, float val );
      float getSpd( int ind );
      int setSLPres( int ind, float val );
      float getSLPres( int ind );
      int setPrec( int ind, float val );
      float getPrec( int ind );

      int print();

      friend class MosUFile;
      friend class MosDataTool;
   };
}   
#endif
