#ifndef _WINC_NAMECONV
#define _WINC_NAMECONV

#include <WXP/Date.h>
#include <WXP/DateVal.h>
#include <WXP/NameConvFile.h>
#include <WXP/String.h>

namespace WXP {
   class NameConv {
      char file[80];
      char hdr[80];
      String path;
      char ext[30];
      char key[20];
      char keyval[20][30];
      int numkey;
      int min_size;
      int min_time;
      char type[20];
      char param[50];
      int off;
      Date date;        // Actual date of file
      DateVal fix_date; // Hard code date components ie using only 12Z data
      Date late_date;   // Start date to be used for searching files
      int fore_time;
      bool output;
      int limit;
      int cpath;
      int npath;
      bool pexist;

      int parsePath( const char *path, const char *conv );
   
      public:
      NameConv( const char *type );
      NameConv();
      ~NameConv();
      int initParam();
      int init();
      int set( const char *type );
      int setOutput( bool val );
      int setFixDate( const char *str );
      int setFixDate( DateVal &rdate );
      inline int getFixDate( DateVal &rdate ){ rdate = fix_date; return 1; };
      int setLateLimit( int val );
      int setLateDate( Date rdate );
      /** Sets the time offset between file times */
      inline int setOffset( int val ){ off = val; return 1; };
      /** Returns the time offset between file times */
      inline int getOffset(){ return off; };
      /** Returns the min size of file used in latest */
      inline int getMinSize(){ return min_size; };
      /** Returns the min time of file used in latest */
      inline int getMinTime(){ return min_time; };
      inline const char *getType(){ return type; };
      /** Sets the forecast time */
      inline int setForeTime( int val ){ fore_time = val; return 1; };
      inline int getForeTime(){ return fore_time; };
      const char *getFile();
      int setExt( const char *str );
      const char *getExt();
      int setKey( char ch, const char *str );
      const char *getKey( char ch );
      int setPath( const char *path );
      int setPath( String &path );
      const char *getPath();
      inline int getNumPath(){ return npath; };
      inline int setPathNum( int ind ){ cpath = ind; return 1; };
      int encode( Date &rdate, int flag, String &filename );
      int encodeFile( Date &rdate, String &filename );
      int encodeHdr( Date &rdate, String &filename );
      int decode( const char *filename );
      int setLatest( const char *str, NameConvFile &file );
      int setFile( Date &rdate, NameConvFile &file );
      int getDate( Date &rdate );
      int print();
   
      static int setOffset( const char *file );

      enum Type { 
	 FILENAME,    /**< Regular filename */
	 HEADER       /**< Header filename */
      };
   };
}   
#endif
