#ifndef _WINC_NETCDFFILE
#define _WINC_NETCDFFILE

#include "netcdf.h"

#include <WXP/FloatArray.h>
#include <WXP/IntArray.h>

namespace WXP {
   class NetCdfFile {
      int fd;
      int shape[10];
      size_t start[10];
      size_t count[10];
      char str[1000];
      int ndim;
      bool opened;
   
      public:
      int init();
      NetCdfFile();
      NetCdfFile( const char *filename, int access );
      NetCdfFile( const char *filename );
      int setFD( int val );
      int printErrors( int val );
      int open( const char *filename, int access );
      int open( const char *filename );
      int defDim( const char *name, int size );
      int defUnlimitDim( const char *name );
      int endDef();
      int close();
      int getNumDim();
      int getNumVar();
      int getNumAttr();
      int getUnlimitDim();
      int getDim( const char *name );
      const char *getDimName( int dim );
      int getDimSize( int dim );
      int setShape( int num, ... );
      int defVar( const char *name, int type, int ndims, ... );
      int defVar( const char *name, int type, IntArray & dims );
      int defFloatVar( const char *name );
      int defIntVar( const char *name );
      int defShortVar( const char *name );
      int defByteVar( const char *name );
      int defCharVar( const char *name );
      int getVar( const char *name );
      const char *getVarName( int var );

      int putGlobAttr( const char *name, const char *value );
      const char *getGlobAttr( const char *name );
      const char *getGlobAttrName( int ind );
      int putBaseAttr( int id, const char *long_name, const char *units );
      int putRangeAttr( int id, float min, float max );
      int putRangeAttr( int id, int min, int max );
      int putFillAttr( int id );
      int putFillAttr( int id, int val );
      int getNumVarAttr( int var );
      const char *getAttr( int var, const char *name );
      const char *getAttrName( int var, int ind );
      int getNumVarDims( int var );
      int getVarDims( int var, IntArray &dims );
      int getVarType( int var );

      int setPos( int num, ... );
      int setPos( IntArray &st );
      int setRange( int num, ... );
      int setRange( IntArray &st, IntArray &cnt );
      int put( int var, IntArray &st, const char *str );
      int putStr( int var, const char *str );
      int put( int var, float value );
      int put( int var, float *array );
      int put( int var, int value );
      int put( int var, short value );
      int put( int var, char value );
      int put( int var, const char *array );
      const char *get( int var, IntArray &st );
      int getRow( int var, IntArray &st, FloatArray &array );
      int getStr( int var, int len, char *str );
      float getFloat( int var );
      int getFloatArray( int var, float *array );
      double getDouble( int var );
      int getDoubleArray( int var, double *array );
      char getChar( int var );
      int getInt( int var );
      char getByte( int var );
      int getByteArray( int var, char *array );

      enum Type {
	 CHAR,
	 BYTE,
	 SHORT,
	 INT,
	 FLOAT,
	 DOUBLE 
      };

   };
}   
#endif
