#ifndef _WINC_NIDSDATA
#define _WINC_NIDSDATA

#include <WXP/Date.h>
#include <WXP/Domain.h>

namespace WXP {
   /** \struct NidsData NidsData.h "../include/NidsData.h"
    *  \brief This struct stores NIDS data into an array
    *
    *  \author Dan Vietor
    *  \ingroup Data
    */
   struct NidsData {
      char info[50];	/** Descriptive string for product */
      char misc[50];	/** Additional information */
      int src;		/** Source of data */
      float lat;	/** Latitude of site */
      float lon;	/** Longitude of site */
      int mode;		/** Operating mode (precip/clear air) */
      int code;		/** Product code (bref, vel, vil) */
      int type;		/** Type */
      int vcp;
      int seq;
      Date date;
      Domain domain;
      int nbin;
      float *data;
      int data_size;

      NidsData();
      ~NidsData();
      int initClass();

      inline int tran( EarthPoint &ept, GridPoint &gpt ){ return domain.tran( ept, gpt); };
      inline float getLat(){ return domain.getPLat(); };
      inline float getLon(){ return domain.getPLon(); };
      inline float getNx(){ return domain.getNx(); };
      inline float getNy(){ return domain.getNy(); };
      inline float getDx(){ return domain.getDx(); };
      inline float getDy(){ return domain.getDy(); };
      int offset( float val );
      float getValue( int x, int y );
      float getValue( EarthPoint &ept );
      int print();

      enum Type { 
	 RADIAL, 
	 RASTOR 
      };

      friend class NidsFile;
      friend class NidsImage;
   };
}
#endif
   
