#ifndef _WINC_NIDSINFO
#define _WINC_NIDSINFO

#include <WXP/Date.h>

namespace WXP {
   /** \class NidsInfo NidsInfo.h "../include/NidsInfo.h"
    *  \brief This structure stores NIDS header information
    *
    *  \author Dan Vietor
    */
   struct NidsInfo {
      char info[50];	/** Descriptive string for product */
      char misc[50];	/** Additional information */
      int src;		/** Source of data */
      float lat;	/** Latitude of site */
      float lon;	/** Longitude of site */
      int mode;		/** Operating mode (precip/clear air) */
      int code;		/** Product code (bref, vel, vil) */
      int type;		/** Type */
      int vcp;
      int seq;
      Date date;	/** Volume scan date*/
      int prod_date;
      int mess_date;
      int nx;
      int ny;
      float res;
      float cres;
      float alt;
      float thres[16];
      float elev;
      float elevt;
      float range;
      float max;
      float nmax;
      int beg_date;
      int beg_time;
      int end_date;
      int end_time;

      NidsInfo();
      int init();
      inline const char *getInfo(){ return info; };
      inline const char *getMisc(){ return misc; };
      inline int getSrc(){ return src; };
      inline float getLat(){ return lat; };
      inline float getLon(){ return lon; };
      inline int getMode(){ return mode; };
      inline int getCode(){ return code; };
      inline int getVcp(){ return vcp; };
      inline int getSeq(){ return seq; };
      inline int getDate( Date &rdate ){ rdate = date; return 1; };
      inline int getDate(){ return date.getSeconds(); };
      inline int getVolDate(){ return date.getSeconds(); };
      inline int getProdDate(){ return prod_date; };
      inline int getMessDate(){ return mess_date; };
      inline float getRes(){ return res; };
      int print();
   };

}
#endif

