#ifndef _WINC_NIDSPLOT
#define _WINC_NIDSPLOT

#include <WXP/Date.h>
#include <WXP/Datum.h>
#include <WXP/Domain.h>
#include <WXP/Drawing.h> 
#include <WXP/Filename.h>
#include <WXP/Listener.h>
#include <WXP/NidsData.h>
#include <WXP/NidsFile.h>
#include <WXP/NidsInfo.h>
#include <WXP/Plot.h>
#include <WXP/StormAttr.h>
#include <WXP/String.h>
#include <WXP/Timezone.h>
#include <WXP/VarList.h>

namespace WXP {

   class NidsPlot : public Drawing, public Listener {
      char id[5];
      char type[15];
      int plt_site;
      int plt_table;
      float plt_dist[5];
      float plt_azimuth;
      bool draw_plot;
      bool draw_data;
      String variable;
      String color_table;
      String color_fill;
      String color;
      int map;
      int clutter_rad;
      int ramp_cut;
      int ramp_len;
      int sync;
      Filename filename; 
      Timezone tz;
      NidsFile nids;
      NidsInfo info;
      NidsData ndata;
      VarList vars;
   
      public:
      NidsPlot();
      NidsPlot( const char *str );
      int init();
      int setParam( const char *str );
      int setPath( const char *path );
      int setFile();
      int setInput( const char *type );
      int setNameConv( const char *type );
      int setHour( const char *str );
      int setFilename();
      int setFilename( Filename &rfile );
      int setFilename( const char *file );
      int setFilename( Date &date );
      int setFilename( Date &date, const char *param );
      inline int setForeTime( const char *fore ){ return 1; };
      int getDate( Date &date );
      int getInitDate( Date &date );
      int getDateOffset();
      int getLoopType();
      int getNumFrames();
      int getDomain( Domain &domain );
      int getInfo( NidsInfo &rinfo );
      int setId( const char *str );
      int setType( const char *str );
      int setVariable( const char *str );
      int setColorTable( const char *str );
      int setColorFill( const char *str );
      int setColor( const char *str );
      inline int setMap( bool val ){ map = val; return 1; };
      inline int getSync(){ return sync; };
      int drawImage( Plot &plot );
      int drawData( Plot &plot );
      const char *getUnits( const char *name );
      int drawStorm( Plot &plot, StormAttr &storm );
      int draw( Plot &plot );
      void callback( const char *ev_str, const char *ev_data, const void *data );
      int print();
   };
}
#endif
