#ifndef _WINC_PARSE
#define _WINC_PARSE

#include <WXP/File.h>
#include <WXP/Filename.h>
#include <WXP/String.h>

namespace WXP {
   class Parse {
      Filename filename;
      int ind;
      File file;
      File hfile;
      bool opened;
     
      bool use_hfile;
      bool use_awips;
   
      char sheader[80];
      char sawip[80];
      char sid[80];
   
      char header[80];
      char awip[80];
      char data[500];  // Temporary data needed for processing
   
      int pline_status;
      int line_status;
      bool next_hdr;   // Need to go to next header
      bool new_prod;   // Is this the beginning of a new product
   
      int loc;    // Location of beginning of current line
      int hloc;   // Location of current header in file
   
      int found;  // Is product found
      int ifound; // Is requested Id found
   
      int end_dollar;
      int end_blank;
   
      int lnum;
   
      bool compare( const char *pattern );
      int newFile();

      public:
      static const int START  = 0x1;
      static const int SEQ    = 0x2;
      static const int HEADER = 0x3;
      static const int AWIP   = 0x4;
      static const int SDATA  = 0x5;
      static const int DATA   = 0x10;
      static const int BLANK  = 0x11;
      static const int DOLLAR = 0x12;
      static const int LOCATE = 0x13;
      static const int VTEC   = 0x14;
      static const int LATLON = 0x15;
      static const int DESC   = 0x16;
   
      Parse();
      Parse( const char *str );
      int init();
      int setParam( const char *str );
      int open( Filename &filename );
      int open( const char *filename );
      bool isOpen();
      int rewind();
      int close();
      /** Determines if header file is to be used in parsing
       * \param val	True if header file is to be used.
       */
      inline int useHdrFile( bool val ){ use_hfile = val; return 1; };
      inline int useAwipsHdr( bool val ){ use_awips = val; return 1; };
      bool isNewProd();
      int setHeader( const char *string );
      int setId( const char *id );
      const char *getHeader();
      const char *getAwip();
      inline int getLineStatus(){ return line_status; };
      int tell();
      int seek( int loc );
      int getLoc();
      int getHeaderLoc();
      int getFileInd();
      int setLoc( int byte );
      int setLoc( int rind, int byte );
      int getLine( char *string, int len );
      int getLine( String &line );
      bool isData();
   
      static bool compare( const char *header, const char *awip, const char *pattern );
      static bool checkZone( const char *line, const char *str );
      enum Type { RAW, WXP };
   };
}
#endif
