#ifndef _WINC_POSTCRIPT
#define _WINC_POSTCRIPT

#include <WXP/Device.h>
#include <WXP/File.h>
#include <WXP/GeomBox.h>
#include <WXP/Image.h>
#include <WXP/Listener.h>
#include <WXP/Point.h>
#include <WXP/Points.h>
#include <WXP/Rect.h>
#include <WXP/String.h>

namespace WXP {
   class Postscript : public Device, public Listener {
      File file;
      bool inverse;
      float width, height;
      float x, y;
      int fixed;
      bool icon;
      bool opened;
      char name[30];
      char filename[60];
      char title[60];
   
      int append;
      int eps;
      int color;
      int orient;
      float scale;
   
      int fore_color;       
      int back_color;   
   
      GeomBox clip;
   
      float base_width;
      int line_bold;
   
      int line_color;
      int line_style;
      float line_width;
   
      int text_color;
      float text_width;
   
      int fill_color;
      int fill_style;
   
      int mark_color;
      float mark_width;
   
      public:
      Postscript();
      ~Postscript();
      int init();
      int setParam( const char *str );
      int open();
      int getDevice( String &str );
      int resize(float rwidth, float rheight);
      int manage();
      int unmanage();
      int addCallback( const char *event, Listener *list, const void *data );
      int doCallback( const char *ev_str, const char *ev_data );
      void callback( const char *ev_str, const char *ev_data, const void *data );
      int close();
      int setFocus();
      int unsetFocus();
      int update();
      int clear();
      int clearRect( Rect &rect );
      int setTitle(const char *str);
      int setIcon(bool val);
      int setFixed(bool val);
      int setInverse(bool val);
      bool getInverse();
      int setBackground(const char *str);
      int setSize(float rwidth, float rheight);
      int getSize(float &width, float &height);
      inline float getWidth(){ return width; };
      inline float getHeight(){ return height; };
      int setOffset(float rx, float ry);
      int getOffset(float &x, float &y);
      int getCoords( Rect &rect );
      Rect getCoords();
      int allocColor( float red, float grn, float blu );
      int setColor( int type, int ind );
      int outputColor( int color );
      int setLineWidth( int type, float width );
      int setLineStyle( int type, int style );
      int setFillStyle( int type, int style );
      int drawLine( int type, float x1, float y1, float x2, float y2 );
      int drawLine( int type, Points &pts );
      int drawFill( int type, Points &pts );
      int setImageAlign( int h, int v );
      int drawImage( Image &image, float x, float y );
      int drawImage( Image &image, int color, float x, float y );
      int getImage( Image &image );
      int setFont( const char *name );
      int setTextAlign( int h, int v );
      int drawText( Point &pt, const char *str );
      Rect getClip();
      int setClip( float x1, float y1, float x2, float y2 );
      int setClip( Rect rect );
      int setCursor( int type );
      int copyFrame( int dest, int src );
      int displayFrame( int ind );
      int clearFrame( int ind );
      int newFrame( int ind );
      int newFrame();
      bool isLooping();
      int getFrame();
      int setFrame( int ind );
      int deleteFrame( int ind );
      int incFrame( int step );
      int getNumFrames();
      int message( const char *str );
      int bell( int len );
   };
}   
#endif
