#ifndef _WINC_RCMDATA
#define _WINC_RCMDATA

#include <WXP/Const.h>
#include <WXP/Date.h>

namespace WXP {
   class RcmData {
      char id[5];
      int num;
      float lat,lon;
      float elev;
      Date date;
      
      int mode;
      int status;
      int loc_type;
      
      char echo[104][104];
      
      int maxt;
      float maxt_x, maxt_y;
      
      struct {
         char id[3];
         float x,y;
         int hgt;
         int dir,spd;
         int hail;
      } storm[40];
      int num_storm;
      
      struct {
         char id[3];
         float x,y;
      } tvs[10];
      int num_tvs;
      
      struct {
         char id[3];
         float x,y;
      } meso[20];
      int num_meso;
      
      struct {
         int hgt;
         int dir, spd;
         int rms;
      } vad[20];
      int num_vad;
   
      public:
      RcmData();
      int init();
      inline const char *getId(){ return id; };
      int setLoc( float rlat, float rlon );
      inline float getLat(){ return lat; };
      inline float getLon(){ return lon; };
      inline Date getDate(){ return date; };
      inline int getMode(){ return mode; };
      inline int getStatus(){ return status; };
      inline int setLocType( int val ){ loc_type = val; return 1; };
      inline int getLocType(){ return loc_type; };
      inline int getMaxTop(){ return maxt; };
      inline float getMaxTopX(){ return maxt_x; };
      inline float getMaxTopY(){ return maxt_y; };
      inline int getNumStorm(){ return num_storm; };
      inline const char *getStormId(int ind){ return ind < 0 || ind >= num_storm ? "": storm[ind].id; };
      inline float getStormX(int ind){ return ind < 0 || ind >= num_storm ? Const::MISS: storm[ind].x; };
      inline float getStormY(int ind){ return ind < 0 || ind >= num_storm ? Const::MISS: storm[ind].y; };
      inline int getStormHgt(int ind){ return ind < 0 || ind >= num_storm ? Const::MISS: storm[ind].hgt; };
      inline int getStormDir(int ind){ return ind < 0 || ind >= num_storm ? Const::MISS: storm[ind].dir; };
      inline int getStormSpd(int ind){ return ind < 0 || ind >= num_storm ? Const::MISS: storm[ind].spd; };
      inline int getStormHail(int ind){ return ind < 0 || ind >= num_storm ? Const::MISS: storm[ind].hail; };
      inline int getNumTvs(){ return num_tvs; };
      inline float getTvsX(int ind){ return ind < 0 || ind >= num_tvs ? Const::MISS: tvs[ind].x; };
      inline float getTvsY(int ind){ return ind < 0 || ind >= num_tvs ? Const::MISS: tvs[ind].y; };
      inline int getNumMeso(){ return num_meso; };
      inline float getMesoX(int ind){ return ind < 0 || ind >= num_meso ? Const::MISS: meso[ind].x; };
      inline float getMesoY(int ind){ return ind < 0 || ind >= num_meso ? Const::MISS: meso[ind].y; };
      int print();
      int printStorm( int ind );
      int printEchoes();
   
      enum LocType {
	 SITE,
	 EARTH
      };

      enum Mode {
         NONE,
	 PRECIP,
	 CLEAR_AIR
      };

      enum Status {
	 NA,
	 NE,
	 OM,
	 PR
      };

      friend class RcmFile;
      friend class RcmWxpFile;
      friend class RcmImage;
      friend class RcmPlot;
      friend class RcmDecode;
      friend class RcmHash;
   };
}   
#endif
