#ifndef _WINC_RCMPLOT
#define _WINC_RCMPLOT

#include <WXP/Date.h>
#include <WXP/DatumPlot.h>
#include <WXP/Domain.h>
#include <WXP/Drawing.h>
#include <WXP/Filename.h>
#include <WXP/Grid.h>
#include <WXP/GridPlot.h>
#include <WXP/Image.h>
#include <WXP/ImagePlot.h>
#include <WXP/Lev2Plot.h>
#include <WXP/Listener.h>
#include <WXP/NidsPlot.h>
#include <WXP/Plot.h>
#include <WXP/RcmData.h>
#include <WXP/RcmDomain.h>
#include <WXP/String.h>
#include <WXP/Timezone.h>
#include <WXP/Unit.h>
#include <WXP/VarList.h>

namespace WXP {
   class RcmPlot : public Drawing, public Listener {
      Filename filename;
      Date date;
      int type;
      int fill;
      bool draw_plot;
      bool draw_data;
      String color_table;
      String color_fill;
      String color;
      String variable;
      char id[5];
      RcmDomain rdom;
      DatumPlot dplot;
      GridCont gcont;
      ImagePlot img;
      int plt_site;
      float plt_dist[5];
      char cutoff[20];
      int ramp_prec_cut;
      int ramp_prec_len;
      int ramp_clar_cut;
      int ramp_clar_len;
      int clutter;
      int filter;
      int min_hgt;
      int num_plt;
      int no_clair;
      Timezone tz;
      VarList vars;
      Unit unit;
      int num;
      int sync;
   
      public:
      RcmPlot();
      RcmPlot( const char *str );
      int init();
      int setParam( const char *str );
      int setPath( const char *path );
      int setFile();
      int setInput( const char *rtype );
      int getType( const char *rtype );
      int setNameConv( const char *name );
      int setHour( const char *str );
      int setFilename();
      int setFilename( const char *name );
      int setFilename( Date &date );
      int setFilename( Date &date, const char *param );
      inline int setForeTime( const char *fore ){ return 1; };
      int setColorTable( const char *str );
      int setColor( const char *str );
      int setColorFill( const char *str );
      int setVariable( const char *str );
      int setMap( bool val );
      int setId( const char *str );
      int getDate( Date &date );
      int getInitDate( Date &date );
      int getDateOffset();
      int getLoopType();
      int getNumFrames();
      int getDomain( Domain &domain );
      inline int getSync(){ return sync; };
      int draw( Plot &plot, Grid &grid ); 
      int draw( Plot &plot, Image &image ); 
      int draw( Plot &plot ); 
      const char *getUnits( const char *name );
      int toImage( RcmData &rdata, Image &image );
      int getData( RcmData &storm, VarSpec &spec, Datum &datum );
      int drawData( Plot &plot, RcmData &rdata );
      void callback( const char *ev_str, const char *ev_data, const void *data );
      int print();

      enum Type {
	 SITE,
	 MOS,
	 MOSG
      };
   };
}
#endif
