#ifndef _WINC_SATFILE
#define _WINC_SATFILE

#include <WXP/AreaFile.h>
#include <WXP/Date.h>
#include <WXP/File.h>
#include <WXP/Filename.h>
#include <WXP/GifFile.h>
#include <WXP/GiniFile.h>
#include <WXP/Image.h>
#include <WXP/ImgWxpFile.h>
#include <WXP/LritFile.h>
#include <WXP/PngFile.h>
#include <WXP/SatUniFile.h>
#include <WXP/TiffFile.h>

namespace WXP {
   class SatFile {
      int type;
      int subtype;
      int ver;
      int access;
      long pos;
      int cnt;
      File file;
      ImgWxpFile img;
      GiniFile gini;
      AreaFile area;
      SatUniFile unisys;
      LritFile lrit;
      TiffFile tiff;
      GifFile gif;
      PngFile png;
      bool opened;
      Date date;
   
      public:
      SatFile();
      SatFile( const char *filename, int raccess, int rtype );
      SatFile( const char *filename, int raccess, const char *rtype );
      SatFile( Filename &filename, int raccess );
      ~SatFile();
      int init();
      int open( const char *filename, int raccess, int rtype );
      int open( const char *filename, int raccess, const char *rtype );
      int open( Filename &filename, int raccess );
      static int getType( const char *str );
      bool isOpen();
      int rewind();
      int close();
      int getHeader( Image &data );
      int read( Image &data );
      int read( const char *filename, int type, Image &image );
      int read( const char *filename, const char *type, Image &image );
      int read( Filename &filename, Image &image );
      int write( Image &data );
      int write( const char *filename, int type, Image &image );
      int setMapRemove( int val ); 
      int setDomain( const char *str ); 
      int setCalibration( const char *str ); 
   
      static int checkType( const char *filename );
      /*
         Satellite types
      */
      enum Type {
         UNK,
         RAW,
         IMG,
         IMGZ,
         GINI,
         GINIZ,
         UNISYS,
         AREA,
	 TIFF,
	 LRIT,
	 GIF,
	 PNG
      };
   };
}   
#endif
