#ifndef _WINC_SFCWMODATA
#define _WINC_SFCWMODATA

#include <WXP/Date.h>
#include <WXP/StrLib.h>

namespace WXP {
   class SfcWmoData {
      char id[11];
      char data[400];
      Date date;
      int pday;
      int phour;
      int pmin;
      int format;
      int location;
      char wmo[21];

      public:
      SfcWmoData();
      int init();

      inline int setId( const char *rid ){ StrLib::copy( id, 11, rid ); return 1; }
      inline const char *getId(){ return id; }

      inline int setData( const char *rdata ){ 
	 StrLib::copy( data, 400, rdata ); return 1; }
      inline const char *getData(){ return data; }

      inline int setDate( Date &rdate ){ date = rdate; return 1; }
      inline Date getDate(){ return date; }

      inline int setProdDay( int rpday ){ pday = rpday; return 1; }
      inline int getProdDay(){ return pday; }

      inline int setProdHour( int rphour ){ phour = rphour; return 1; }
      inline int getProdHour(){ return phour; }

      inline int setProdMinute( int rpmin ){ pmin = rpmin; return 1; }
      inline int getProdMinute(){ return pmin; }

      inline int setFormat( int rformat ){ format = rformat; return 1; }
      inline int getFormat(){ return format; }

      inline int setLocation( int rloc ){ location = rloc; return 1; }
      inline int getLocation(){ return location; }

      inline int setWmo( const char *rwmo ){ 
	 StrLib::copy( wmo, 21, rwmo ); return 1; }
      inline const char *getWmo(){ return wmo; }

      int printData();
      int print();

      friend class SfcWmoFile;
      friend class MetarTool;
      friend class SfcDecode;

      static inline bool isMETAR( int type ){ return type >= METAR && type <= SPECI ? true : false; };
      static inline bool isSA( int type ){ return type >= SA && type <= SP ? true : false; };
      static inline bool isSYNOP( int type ){ return type >= SYNOP && type <= CMAN ? true : false; };
      static inline bool isSPEC( int type ){ return type == SPECI || type == SP ? true : false; };

      enum Location {
	 UNK,
	 USA,
	 CANADA,
	 MEXICO,
	 OTHER
      };

      enum DataType { 
	 NONE, 
	 METAR, 
	 SPECI, 
	 SA, 
	 SP, 
	 SYNOP, 
	 SHIP, 
	 BUOY, 
	 DRIBU, 
	 MOBIL, 
	 CMAN, 
	 MISC 
      };
   };
}

#endif
