#ifndef _WINC_UPACALC
#define _WINC_UPACALC

#include <WXP/BrnData.h>
#include <WXP/CclData.h>
#include <WXP/HelicityData.h>
#include <WXP/ParcelMeanData.h>
#include <WXP/ShearData.h>
#include <WXP/ThetaEData.h>
#include <WXP/UpaData.h>
#include <WXP/UpaLevel.h>
#include <WXP/VCoord.h>
#include <WXP/Wind.h>

namespace WXP {
   class UpaCalc {
      public:
      static float calc( UpaLevel &level, const char *var ); 
      static const char *getUnits( const char *var ); 
      static int thickTop( VCoord &bottom, float thick, VCoord &top );
      static VCoord thickTop( VCoord &bottom, float thick );
      static int thickBottom( VCoord &top, float thick, VCoord &bottom );
      static VCoord thickBottom( VCoord &top, float thick );
      static int parcelMean( UpaData &data, VCoord &bottom, 
      	    VCoord &top, ParcelMeanData &pmean );
      static ParcelMeanData parcelMean( UpaData &data, VCoord &bottom, 
      	    VCoord &top );
      static int meanWind( UpaData &data, VCoord &bottom, VCoord &top, Wind &wind );
      static Wind meanWind( UpaData &data, VCoord &bottom, VCoord &top );
      static float meanValue( UpaData &data, const char *type, 
      	    VCoord &bottom, VCoord &top );
      static int ccl( UpaData &data, VCoord &bottom, float thick, CclData &ccl );
      static CclData ccl( UpaData &data, VCoord &bottom, float thick );
      static float thickness( UpaData &data, VCoord &bottom, VCoord &top );
      static float lapseRate( UpaData &data, VCoord &bottom, VCoord &top );
      static float thetaLapseRate( UpaData &data, VCoord &bottom, 
	    VCoord &top );
      static float thetaPresLapseRate( UpaData &data, VCoord &bottom, 
	    VCoord &top );
      static float staticStability( UpaData &data, VCoord &level );
      static float precipWater( UpaData &data );
      static float freezingLevel( UpaData &data );
      static float wetBulbZero( UpaData &data );
      static float maxTemperature( UpaData &data, VCoord &bottom, 
	    float thick );
      static float energyCalc( UpaLevel &par1, UpaLevel &par2, UpaLevel &env1, 
      	    UpaLevel &env2 );
      static float capStrength( UpaData &data, VCoord &bottom, 
	    float thick );
      static float liftedIndex( UpaData &data, VCoord &bottom,
	    float thick, VCoord &top );
      static float totalTotals( UpaData &data );
      static float verticalTotals( UpaData &data );
      static float crossTotals( UpaData &data );
      static float kIndex( UpaData &data );
      static float sweatIndex( UpaData &data );
      static float energyIndex( UpaData &data );
      static int thetaEIndex( UpaData &data, ThetaEData &thetae );
      static ThetaEData thetaEIndex( UpaData &data );
      static int helicity( UpaData &data, VCoord &bottom, 
	    VCoord &top, Wind &storm, HelicityData &hel );
      static HelicityData helicity( UpaData &data, VCoord &bottom, 
	    VCoord &top, Wind &storm );
      static int shearLayer( UpaData &data, VCoord &bottom, 
	    VCoord &top, ShearData &shear );
      static ShearData shearLayer( UpaData &data, VCoord &bottom, 
	    VCoord &top );
      static int bulkRichardson( UpaData &data, VCoord &bottom, 
      	    float cape, BrnData &brn );
      static BrnData bulkRichardson( UpaData &data, VCoord &bottom, 
      	    float cape );
      static float energyHelicity( UpaData &data, VCoord &bottom, 
      	    Wind &storm, float cape );

   };
}
#endif

