#ifndef _WINC_UPADATA
#define _WINC_UPADATA

#include <WXP/Date.h>
#include <WXP/StrLib.h>
#include <WXP/UpaLevel.h>
#include <WXP/VCoord.h>

namespace WXP {
   /*
      Upper air merged data structure
   */
   class UpaData {
      int type;                 /* Observation type */
      Date date;                /* Observation time */
      int fhour;                /* Forecast hour */
      char gen_name[20];        /* Generation name */
      char id[20];              /* Station identifier */
      char area[20];            /* Regional identifier */
      char wmo[6];              /* WMO station number */
      float lat;                /* Station latitude */
      float lon;                /* Station longitude */
      float elev;               /* Station elevation */
      public:
      static const int MAXLEV = 300;
   
      UpaLevel lev[MAXLEV];     /* Level information */
      int num;                  /* Number of levels */
      int sfc;                  /* Index of surface level */
      int maxwnd;               /* Index of max wind level */
      int trop;                 /* Index of tropopause level */
   
      public:
      UpaData();
      int init();
      int print();
      int addLevel( UpaLevel &level );
      int delLevel( int ind );
      int updateLevel( int ind, UpaLevel &level );
      int setNum( int val );
      bool isSfc();
      bool isLevelMandatory( int ind );
      int sortPres();
      int sortHeight();
      int calcHeight();
      int calcPres();
      int interpTemps();
      int interpWinds();
      int interp( int coord, float val, UpaLevel &level );
      int interp( VCoord coord, UpaLevel &level );

      inline int setType( int rtype ){ type = rtype; return 1; };
      inline int getType(){ return type; };
      int setId( const char *rid );
      inline const char *getId(){ return id; };
      int setArea( const char *rarea );
      inline const char *getGenName(){ return gen_name; };
      int setGenName( const char *str );
      inline int setFhour( int val ){ fhour = val; return 1; };
      inline int getFhour(){ return fhour; };
      inline const char *getArea(){ return area; };
      inline int setDate( Date &rdate ){ date = rdate; return 1; };
      inline Date getDate(){ return date; };
      inline int getDate( Date &rdate ){ rdate = date; return 1; };
      inline int setWmo( const char *rwmo ){ StrLib::copy( wmo, 6, rwmo ); return 1; };
      inline const char *getWmo(){ return wmo; };
      inline int setLoc( float rlat, float rlon ){ lat = rlat; lon = rlon; return 1; };
      inline float getLat(){ return lat; };
      inline float getLon(){ return lon; };
      int setElev( float relev );
      inline float getElev(){ return elev; };

      inline int setSfc( int ind ){ sfc = ind; return 1; };
      inline int getSfc(){ return sfc; };
      inline int setMaxWnd( int ind ){ maxwnd = ind; return 1; };
      inline int getMaxWnd(){ return maxwnd; };
      inline int setTrop( int ind ){ trop = ind; return 1; };
      inline int getTrop(){ return trop; };

      int getSfc( UpaLevel &lev );
      float getSfcPres();
      int setSfcHght( float val );
      float getSfcHght();
      int getTrop( UpaLevel &lev );
      float getTropPres();
      int getMaxWnd( UpaLevel &lev );
      float getMaxWndPres();

      inline int getNum(){ return num; };
      UpaLevel getLevel( int level );
      int setType( int level, int val );
      int getType( int level );
      int setPres( int level, float val );
      float getPres( int level );
      int setHght( int level, float val );
      float getHght( int level );
      int setTemp( int level, float val );
      float getTemp( int level );
      float getTempK( int level );
      int setDewp( int level, float val );
      float getDewp( int level );
      float getDewpK( int level );
      int setSpd( int level, float val );
      float getSpd( int level );
      int setDir( int level, float val );
      float getDir( int level );
      bool validPres( int level );
      bool validHght( int level );
      bool validTemp( int level );
      bool validDewp( int level );
      bool validDir( int level );
      bool validSpd( int level );

      friend class UpaDataTool;
   };
}
#endif

