#ifndef _WINC_UPALEVEL
#define _WINC_UPALEVEL

namespace WXP {
   struct UpaLevel {
      int type;                 /**< Type of level */
      float p;                  /**< Pressure */
      float h;                  /**< Height */
      float t;                  /**< Temperature */
      float td;                 /**< Dewpoint */
      float dir;                /**< Wind direction */
      float spd;                /**< Wind speed */

      int set( float rp, float rh, float rt, float rtd, float rdir, float rspd );
      /** Sets the pressure */
      inline int setPres( float v ){ p = v; return 1; }
      /** Gets the pressure */
      inline float getPres(){ return p; }
      /** Sets the geopotential height */
      inline int setHght( float v ){ h = v; return 1; }
      /** Gets the geopotential height */
      inline float getHght(){ return h; }
      /** Sets the temperature */
      inline int setTemp( float v ){ t = v; return 1; }
      /** Gets the temperature */
      inline float getTemp(){ return t; }
      /** Sets the dewpoint */
      inline int setDewpt( float v ){ td = v; return 1; }
      /** Gets the dewpoint */
      inline float getDewpt(){ return td; }
      /** Sets the wind direction */
      inline int setWDir( float v ){ dir = v; return 1; }
      /** Gets the wind direction */
      inline float getWDir(){ return dir; }
      /** Sets the wind speed */
      inline int setWSpd( float v ){ spd = v; return 1; }
      /** Gets the wind speed */
      inline float getWSpd(){ return spd; }
      /** Sets the level type \see MAN, SIG, SFC, etc */
      inline int setType( int v ){ type = v; return 1; }
      /** Gets the level type */
      inline int getType(){ return type; }

      static const int NONE = 0;  /**< No level type */
      static const int MAN = 1;   /**< Mandatory level */
      static const int SIG = 2;   /**< Significant level */
      static const int WND = 3;   /**< Wind level */
      static const int SFC = 0x04;  /**< Surface level */
      static const int TROP = 0x10; /**< Tropopause level */
      static const int MWND = 0x20; /**< Max wind level */
      static const int LCL = 0x40;  /**< LCL level */

      UpaLevel();
      int init();
      int update( UpaLevel &level );
      int print();

      static bool isMandatory( float p );
      static int getMandatoryIndex( float p );
   };
}
#endif
