#ifndef _WINC_VARSPEC
#define _WINC_VARSPEC

#include <WXP/Date.h>
#include <WXP/ForeTime.h>
#include <WXP/Level.h>
#include <WXP/String.h>
#include <WXP/Timezone.h>
#include <WXP/VarName.h>
#include <WXP/Variable.h>

namespace WXP {
   class VarSpec {
      int type;            /* Type of variable */
      int source;          /* Source of variable */
      bool abs;
      int plot;
      ForeTime fore;       /* Forecast time */
      Level level;         /* Vertical level */
      Variable var;        /* Variable */
      char format[100];
      VarName vname;

      int initResrc();
      public:
      static const int format_len = 100;

      VarSpec();
      VarSpec( const char *str );
      int init();

      int set( const char *str );
      int set( VarSpec &vs );

      /** Sets the variable type */
      inline int setType( int t ){ type = t; return 1; };
      /** Gets the variable type */
      inline int getType(){ return type; };

      /** Sets the plot type */
      inline int setPlot( int t ){ plot = t; return 1; };
      /** Gets the plot type */
      inline int getPlot(){ return plot; };

      inline int setFore( const char *str ){ return fore.set( str ); };
      inline int set( ForeTime &rfore ){ fore = rfore; return 1; };
      inline int get( ForeTime &rfore ){ rfore = fore; return 1; };
      inline ForeTime getFore(){ return fore; };
      inline int getForeType(){ return fore.getType(); };
      inline float getForeHours(){ return fore.getHours(); };
      inline int getForeSeconds(){ return fore.getSeconds(); };
      inline int getForeString( String &str ){ return fore.toString( str ); };
      inline bool isForeSet(){ return fore.isSet(); };

      inline int setLevel( const char *str ){ return level.set( str ); };
      inline int set( Level &rlevel ){ level = rlevel; return 1; };
      inline int get( Level &rlevel ){ rlevel = level; return 1; };
      inline int getLevelType1(){ return level.getType1(); };
      inline Level getLevel(){ return level; };
      inline int getLevelString( String &str ){ return level.toString( str ); };
      inline bool isLevelSet(){ return level.isSet(); };

      int setVar( int ind, const char *unit );
      int setVar( const char *abbr, const char *unit );
      int setVar( const char *str );
      int set( Variable &rvar );
      inline int get( Variable &rvar ){ rvar = var; return 1; };
      inline Variable getVar(){ return var; };

      inline int setUnits( const char *str ){ return var.setUnits( str ); };
      inline const char *getUnits(){ return var.getUnits(); };

      int setFormat( const char *str );
      inline const char *getFormat(){ return format; };

      inline int getForeCode(){ return fore.toCode(); };
      inline int getLevelTypeCode(){ return level.toTypeCode(); };
      inline int getLevelCode(){ return level.toLevelCode(); };
      inline int getVarCode(){ return var.toIndCode(); };
      inline int getVarType(){ return var.getType(); };
      inline int getVarInd(){ return var.getInd(); };
      inline const char *getVarAbbr(){ return var.getAbbr(); };

      int decode( const char *str );
      int toString( String &str );

      inline int toForeLabel( String &str ){ return fore.toString( str ); };
      inline int toLevelLabel( String &str ){ return level.toString( str ); };
      inline int toVarLabel( String &str ){ return var.toString( str ); };
      inline int toVarAbbr( String &str ){ return var.toAbbrCode( str ); };

      int toInfoLabel( String &str );
      int toDateLabel( Date &date, Timezone &tz, const char *src, String &str );
      int toDateLabel( Date &date, const char *src, String &str );

      int update( VarSpec &v1, int flag );
      int update( VarSpec &v1 );
      int update( ForeTime &ft, Level &lev );
      int update( ForeTime &ft );
      int update( Level &lev );
      inline int printFore(){ return fore.print(); };
      inline int printLevel(){ return level.print(); };
      inline int printVar(){ return var.print(); };
      int printDetails();
      int print();

      enum Type { NONE, PARAM, VECT, GROUP, FUNC, MISC };
      enum Plot { DEF, PLOT, DATA, CONTOUR, VECTOR, MAP, PANEL };

      static const int FORETIME = 0x1;
      static const int LEVEL = 0x2;
      static const int VAR =0x4;
      static const int FORMAT =0x8;
      static const int ALL =0xF;

      static int update( VarSpec &v1, VarSpec &v2, int flag );

      friend class VarList;
   };
}
#endif
