#ifndef _WINC_WATCHDATA
#define _WINC_WATCHDATA

#include <WXP/Date.h>
#include <WXP/EarthPoint.h>

namespace WXP {
   struct WatchData {
      int type;		/**< Type of watch */
      int num;		/**< Watch number */
      int status;
      int replaces;
      Date sdate;
      Date edate;
      float width;      /**< Box width */
      int axis;         /**< Axis direction */
      char id[4][4];         /* Two endpoint station IDs */
      EarthPoint ept[6];     /* Endpoint station latitudes */
      float ang[4];          /* Direction and distance from endpoints */
      float dist[4];

      /** Watch type */
      enum Type { 
	 UNK,      /**< Unknown type */
	 SEVERE,   /**< Severe thunderstorm watch */
	 TORNADO,  /**< Tornado watch */
	 TEST      /**< Test */
      }; 
      /** Axis direction */
      enum Axis { 
	 EW,       /**< East-West */
	 NS,       /**< North-South */
	 ES        /**< Either side */
      };   
      /** Status */
      enum Status { 
	 PENDING,   /**< Pending, before valid time */
	 VALID,     /**< Valid */
	 ENDED,     /**< Ended */
	 CANCELLED, /**< Cancelled */
	 REPLACED   /**< Replaced with another watch */
      }; 

      WatchData();
      int init();
      int print();
   };
}
#endif   
