#ifndef _WINC_WATCHPLOT
#define _WINC_WATCHPLOT

#include <WXP/Date.h>
#include <WXP/Domain.h>
#include <WXP/Drawing.h>
#include <WXP/Filename.h>
#include <WXP/Listener.h>
#include <WXP/Plot.h>
#include <WXP/Timezone.h>
#include <WXP/WatchData.h>

namespace WXP {
   class WatchPlot : public Drawing, public Listener {
      int label;            /* Plot labels */
      float scale;
      int shadow;
      int fill;
      Date vdate;
      bool vdate_fix;
      Filename filename;
      String color_table;
      String color;
      WatchData watches[40];
      Timezone tz;
      int num_watch;
      bool map;
      int sync;

      public:
      WatchPlot();
      WatchPlot( const char *param );
      int init();
      int setParam( const char *str );
      int setFile();
      int setInput( const char *type );
      int setHour( const char *str );
      int setNameConv( const char *type );
      int setFilename();
      int setFilename( const char *str );
      int setFilename( Date &date );
      int setFilename( Date &date, const char *param );
      /** Sets the forecast time (ignored) */
      inline int setForeTime( const char *fore ){ return 1; };
      int getDate( Date &date );
      int getInitDate( Date &date );
      int getDateOffset();
      int getLoopType();
      int getNumFrames();
      int getDomain( Domain &domain );
      int setScale( float val );
      int setTimezone( const char *str );
      int setShadow( int val );
      int setValidDate( const char *str );
      int setValidDate( Date &date );
      int setColorTable( const char *str );
      int setColor( const char *str );
      /** Sets whether object is to draw map or not */
      inline int setMap( bool val ){ map = val; return 1; };
      /** Specifies whether object is a time sync */
      inline int getSync(){ return sync; };
      int readFile( const char *filename );
      int draw( Plot &plot );
      void callback( const char *ev_str, const char *ev_data, const void *data );
      int print();
   };
}
#endif
