#ifndef _WINC_XWINDOW
#define _WINC_XWINDOW

#include <X11/Xlib.h>

#include <WXP/Device.h>
#include <WXP/Image.h>
#include <WXP/Listener.h>
#include <WXP/Point.h>
#include <WXP/Points.h>
#include <WXP/Rect.h>
#include <WXP/String.h>

namespace WXP {
   class XWindow : public Device, public Listener {
      struct XFrame {
         char title[50];
         Pixmap pixmap;
         float delay;
         int flag;
      };
   
      struct XCallbackItem {
         int type;
         Window window;
         Listener *listener;
         const void *data;
         struct XCallbackItem *next;
      };
   
      static Display *display;
      static int managed;
      static int num_errors;
      static int error;
      static int error_print;
      static XCallbackItem *callbackroot;
      static XWindow *windows[10];
      static int num_windows;
   
      int type;
      bool inverse;
      int width, height;
      int x, y;
      int fixed;
      bool mapped;
      bool icon;
      bool opened;
      char name[30];
      char connect[40];
      char title[60];
      int window_num;
      int show_win_num;
   
      int screen;              /* X display screen */
      Colormap color_map;      /* Default color map */
      Window window;           /* The base window */
      Window dwindow;          /* The backing pixmap */
      Window drawing;          /* The drawing pixmap */
      Visual *visual;          /* The display visual */
      int depth;               /* The display depth */
   
      Pixmap icon_pixmap;      /* Icon pixmap */
      Cursor cursor_x;         /* X cursor for X */
      Cursor cursor_ptr;       /* X cursor for pointing */
      Cursor cursor_move;      /* X cursor for panning */
      Cursor cursor_loop;      /* X cursor for looping */
      Cursor cursor_stop;      /* X cursor for stop mode */
      Cursor cursor_draw;      /* X cursor for line drawing */
      Cursor cursor_rect;      /* X cursor for rectangle drawing */
      Cursor cursor_text;      /* X cursor for drawing text */
      Cursor cursor_wait;      /* X cursor for waiting */
      int cursor;              /* Current cursor type */
   
      int fore_color;       
      int back_color;   
      char background[40];
      int bg_color;   
   
      XRectangle clip;
   
      GC base_gc;
      GC draw_gc;
      GC line_gc; 
      GC text_gc;
      GC fill_gc;
      GC mark_gc;
      int line_style;
      int line_width;
      Pixmap fill_pat[20];     /* Fill pattern pixmaps */
      int text_halign;
      int text_valign;
      XFontStruct *font_info;
      int image_halign;
      int image_valign;
   
      /* Loop data */
      int max_frame;
      int num_frame;
      int cur_frame;
      int looping;
      int direct;
      float delay;
      float end_delay;
      int start_frame;
      int end_frame;
      XFrame *frames;
   
      int initStatic();
      int printImage( XImage *ximage );

      public:
      XWindow();
      ~XWindow();
      int init();
      int setParam( const char *str );
      int open();
      int getDevice( String &str );
      int resize(float rwidth, float rheight);
      int addCallback( const char *event, Listener *list, const void *data );
      int doCallback( const char *ev_str, const char *ev_data );
      void callback( const char *ev_str, const char *ev_data, const void *data );
      int close();
      int setFocus();
      int unsetFocus();
      int update();
      int clear();
      int clearRect( Rect &rect );
      int manage();
      int unmanage();
      int setTitle( const char *str );
      inline const char *getTitle(){ return title; };
      int setIcon(bool val);
      int setFixed(bool val);
      int setCursor(int type);
      inline int getCursor(){ return cursor; };
      int setInverse(bool val);
      bool getInverse();
      int setBackground( const char *str );
      int setSize(float rwidth, float rheight);
      int getSize(float &width, float &height);
      inline float getWidth() { return width; };
      inline float getHeight() { return height; };
      int setOffset(float rx, float ry);
      int getOffset(float &x, float &y);
      int getCoords( Rect &rect );
      Rect getCoords();
      int allocColor( float red, float grn, float blu );
      int setColor( int type, int ind );
      int setLineWidth( int type, float width );
      int setLineStyle( int type, int style );
      int setFillStyle( int type, int style );
      int drawLine( int type, float x1, float y1, float x2, float y2 );
      int drawLine( int type, Points &pts );
      int drawFill( int type, Points &pts );
      int setImageAlign( int h, int v );
      int drawImage( Image &image, float x, float y );
      int drawImage( Image &image, int bgcolor, float x, float y );
      int getImage( Image &image );
      int setFont( const char *name );
      int setTextAlign( int h, int v );
      int drawText( Point &pt, const char *str );
      Rect getClip();
      int setClip( float x1, float y1, float x2, float y2 );
      int setClip( Rect rect );
      Pixmap getFramePixmap( int ind );
      int copyFrame( int dest, int src );
      int displayFrame( int ind );
      int clearFrame( int ind );
      int newFrame( int ind );
      int newFrame();
      bool isLooping();
      int getFrame();
      int setFrame( int ind );
      int incFrame( int step );
      int deleteFrame( int ind );
      int deleteFrame();
      int getNumFrames();
      int loopRate( int dir );
      int toggleLoop();
      int message( const char *str );
      int printGC( int type );
      int bell( int len );
   
      static int mainloop();
      static int waitEvent();
      static int checkEvent();
      static int addWindow( Window &win );
      static int processEvent( XEvent *event );
      static int procCallback( Window win, int type, const char *ev_str, const char *ev_data );
      static int newCallback( Window win, int event, Listener *list, const void *data );
      static int deleteCallbacks( Window );
      static int errorHandler( Display *dpy, XErrorEvent *error );
      static int errorIOHandler( Display *dpy );
   
      static const int AnyEvent =  0;
      static const int Redraw   = 50;
   
      static const int WINDOW = 1;
      static const int PIXMAP = 2;

      static const int CURSOR_X    = 0;
      static const int CURSOR_PTR  = 1;
      static const int CURSOR_MOVE = 2;
      static const int CURSOR_LOOP = 3;
      static const int CURSOR_STOP = 4;
      static const int CURSOR_DRAW = 5;
      static const int CURSOR_RECT = 6;
      static const int CURSOR_PLUS = 6;
      static const int CURSOR_TEXT = 7;
      static const int CURSOR_WAIT = 8;
   };
}
#endif
