#ifndef _WINC_ZLIB
#define _WINC_ZLIB

#include <zlib.h>

#include <WXP/Buffer.h>

namespace WXP {
   class Zlib {
      int type;
      int setup;
      z_stream zs;
      int stat;

      public:
      Zlib();
      Zlib( int rtype );
      int init();
      int set( unsigned char *next_in, int avail_in, 
      	    unsigned char *next_out, int avail_out );
      int decomp();
      int decomp( Buffer &in, Buffer &out );
      int comp( int flag );
      int finish();
      int print();

      //static void *alloc( void *opaque, unsigned int num, unsigned int size );
      //static void free( void *opaque, void *buf );

      /** Returns status */
      inline int getStatus(){ return stat; };
      /** Returns available bytes not processed */
      inline int getAvailIn(){ return zs.avail_in; };
      /** Returns available bytes out */
      inline int getAvailOut(){ return zs.avail_out; };
      /** Returns total bytes output */
      inline int getTotalOut(){ return zs.total_out; };
      /** Returns if there was an error */
      inline bool isError(){ return stat < 0 ? true : false; };
      
      static int const STREAM_END = Z_STREAM_END; /**< End of stream status */
      static int const NEED_DICT = Z_NEED_DICT; /**< Need dictionary error */
      static int const OK = Z_OK; /**< No error in processing */
      static int const FINISH = Z_FINISH; /**< Finish compression of data */

      /** File type */
      enum Type {
	 COMP,    /**< Compress data */
	 DECOMP   /**< Decompress data */
      };
   };
}
#endif
